# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

from os.path import join as opj

from twisted.cred.error import UnauthorizedLogin
from twisted.cred.portal import Portal, IRealm
from twisted.cred.checkers import ICredentialsChecker
from twisted.cred.credentials import UsernamePassword
from twisted.python.components import Interface, Adapter, registerAdapter

from atop.test import unittest
from atop.powerup import IPowerStation
from atop.credup import StoreCredentialsChecker, StoreRealm, Avatar, ILoginRelationRegistry
from atop.store import Store

from atop.faraway import FarIndex, IFarIndex

class IUnadaptable(Interface):
    pass

class IAvatarTest(Interface):
    """
    Placeholder interface for tests.
    """

class AdapterTest(Adapter):
    __implements__ = IAvatarTest

    value = None

    def dummyMethod(self, value):
        self.original.touch()
        self.value = value

registerAdapter(AdapterTest, Avatar, IAvatarTest)

USERNAME = "userXname"
PASSWORD = "passXword"

def passthrough(a, i):
    pass
def refuseAll(a, i):
    raise Exception("All logins disabled!!!")
    
class LoginExampleAccess(unittest.TestCase):

    storePowerups = [StoreRealm]

    def dbSetUp(self):
        icc = ICredentialsChecker(self.store)
        icc.addDomain("divmod.org")
        self.avatar = self.createAvatar((), name=USERNAME)
        self.avatar.setInsecurePassword(PASSWORD)
        icc.addAvatarToDomain(self.avatar, "divmod.org")

    def testDefaultLogin(self):
        p = Portal(IRealm(self.store))
        p.registerChecker(ICredentialsChecker(self.store))
        r = unittest.deferredResult(p.login(UsernamePassword(
            USERNAME, PASSWORD), None, IAvatarTest))
        self.assertEquals(len(r), 3)
        self.assertEquals(r[0], IAvatarTest)
        self.assertEquals(r[1], IAvatarTest(self.avatar))
        # Who cares about r[2]?

    def testMultipleInterfaces(self):
        r = IRealm(self.store)
        ial = r.requestAvatar(self.avatar.storeID, None, IUnadaptable, IAvatarTest)
        self.assertEquals(ial[0], IAvatarTest)
        self.failUnless(isinstance(ial[1], AdapterTest))
        self.assertEquals(ial[1].original, self.avatar)

        # Just make sure it doesn't raise
        ial[2]()

    def dbtestFarAvatar(self):
        from twisted.internet import reactor
        iffy = IFarIndex(self.avatar).farByName(USERNAME)
        # next line is written in this way to make sure that if I relax the
        # reactor-must-iterate requirement it will still work -gl
        iffy.far(IAvatarTest, "dummyMethod", 7).addBoth(lambda q: (reactor.callLater(0, reactor.crash), q)[-1])
        reactor.run()
        self.assertEquals(IAvatarTest(self.avatar).value, 7)

    def dbtestRelationRefusal(self):
        reg = IRealm(self.store) # ILoginRelationRegistry(self.store)
        
        ial = reg.requestAvatar(self.avatar.storeID, None, IAvatarTest)
        self.assertEquals(ial[0], IAvatarTest)
        self.failUnless(isinstance(ial[1], AdapterTest))
        self.assertEquals(ial[1].original, self.avatar)

        reg.addRefusalRelation(passthrough)
        
        ial = reg.requestAvatar(self.avatar.storeID, None, IAvatarTest)
        self.assertEquals(ial[0], IAvatarTest)
        self.failUnless(isinstance(ial[1], AdapterTest))
        self.assertEquals(ial[1].original, self.avatar)

        reg.addRefusalRelation(refuseAll)

        self.assertRaises(UnauthorizedLogin,
                          reg.requestAvatar, self.avatar.storeID, None, IAvatarTest)
        
