# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

import os
import signal

from os.path import join as opj

from atop import store
from atop.test import unittest

from twisted.python.util import sibpath
from twisted.python import failure
from twisted.internet import defer
from twisted.internet import reactor
from twisted.internet.protocol import ProcessProtocol

class AtopProcess(ProcessProtocol):

    creatingStore = False
    createdStore = False
    
    def __init__(self, lineToMurderOn, d):
        self.buffer = ''
        self.murderOn = lineToMurderOn
        self.doneDeferred = d
        
        self.creatingAvatars = None
        self.createdAvatars = []

        self.creatingItems = None
        self.createdItems = []
    
    def connectionMade(self):
        pass

    def processEnded(self, reason):
        self.doneDeferred.callback(self)
        self.doneDeferred = None
    
    def murder(self):
        os.kill(self.pid, signal.SIGKILL)

    def lineReceived(self, line):
        if line == 'About to create store':
            self.creatingStore = True
        elif line == 'Created the store':
            self.createdStore = True
        elif line.startswith('About to create avatars'):
            self.creatingAvatars = map(int, line.split()[-2:])
        elif line.startswith('Avatar IDs'):
            self.createdAvatars.extend(map(int, line.split()[2:]))
            self.creatingAvatar = None
        elif line.startswith('About to create items for'):
            self.creatingItems = line.split()[-1]
        elif line == 'Item IDs':
            self.createdItems.extend(map(int, line.split()[2:]))
            self.creatingItems = None

        if line == self.murderOn:
            self.murder()

    def outReceived(self, bytes):
        self.buffer += bytes
        lines = self.buffer.splitlines(True)
        for L in lines:
            if L.endswith('\n'):
                self.lineReceived(L[:-1])
            else:
                self.buffer = L
                return
        self.buffer = ''

    def errReceived(self, line):
        self.doneDeferred.errback(failure.Failure(AssertionError("stderr should never be written to (got: %s)" % (line,))))
        self.doneDeferred = None

from sys import executable
atop_executable = sibpath(__file__, "atop_process.py")

from atop.test import atop_process

class RecoveryFromSIGKILL(unittest.TestCase):

    todo = "Work in progress"

    nRuns = 1

    def assertDatabaseConsistency(self, store, proto):
        self.failUnless(proto.creatingStore)
        self.failUnless(proto.createdStore)

        i = 0
        for aid in proto.createdAvatars:
            a = store.getItemByID(aid)
            self.assertEquals(a.name, "Avatar %d (foo)" % i)
            i += 1
            for j in range(atop_process.ITEMS_PER_AVATAR):
                I = a.getItemByID(j + 1)
                self.assertEquals(I.name, "Item %d of Avatar %d" % (j, a.storeID))


    def partEarlyKillInALoop(self):
        path = os.path.abspath(self.mktemp())
        done = defer.Deferred()
        proto = AtopProcess('About to create avatars 0 1', done)
        args = (executable, atop_executable, path)
        env = {'PYTHONPATH': os.environ.get('PYTHONPATH', '')}
        atopProcess = reactor.spawnProcess(proto, executable, args, env)
        proto.pid = atopProcess.pid
        self.assertIdentical(proto, unittest.deferredResult(done))
        
        s = store.Store(0, opj(path, 'db'), opj(path, 'files'))
        
        self.assertDatabaseConsistency(s, proto)

    multitxn = [partEarlyKillInALoop] * nRuns
