# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

from os.path import join as opj

from atop.test import unittest
from atop.sched import SystemScheduler, SubScheduler, SchedulerService, SchedulerBase, Schedulable
from atop.powerup import IPowerStation
from atop.store import Store, SubStore, Item

class MySch(Item, Schedulable):
    count = 0
    def run(self):
        self.count += 1

class SchedTest(unittest.TestCase):

    def dbSetUp(self):
        self.times = [0.0, 1.0, 2.0, 3.0, 10.0, 20.0, 30.0]
        self.calls = []
        SchedulerBase.clock = staticmethod(lambda : self.times[0])
        SystemScheduler.callLater = staticmethod(lambda t, f, *a, **b: self.calls.append((t, f)))

        self.svc = SchedulerService(self.store)
        self.getTopSched = self.svc.ref.getItem
        self.sub = SubStore(self.store)
        IPowerStation(self.sub).installPowerup(SubScheduler)
        self.svc.startService()

    def dbtestTopScheduler(self):
        msc = MySch(self.store)
        msc.schedule(2.5)
        for x in range(10):
            t, f = self.calls.pop(0)
            f()
        self.assertEquals(msc.count, 0)
        for x in range(3):
            self.times.pop(0)
            t, f = self.calls.pop(0)
            f()
        self.assertEquals(msc.count, 1)
        self.assertEquals(self.calls, [])

    def dbtestSubScheduler(self):
        msc = MySch(self.sub)
        msc.schedule(2.5)
        for x in range(10):
            t, f = self.calls.pop(0)
            f()
        self.assertEquals(msc.count, 0)
        for x in range(3):
            self.times.pop(0)
            t, f = self.calls.pop(0)
            f()
        self.assertEquals(msc.count, 1)
        self.assertEquals(self.calls, [])

        self.failIf(list(self.getTopSched().poolRef.getItem().queryIndex()), "Items left in the scheduled pool")
