
import gc

from bsddb.db import DBNoMemoryError

from atop.store import Item, Pool, Store
from twisted.trial import unittest

class TortureTest(unittest.TestCase):
    def setUp(self):
        from atop import store as mangle
        self.oldml = mangle.MAX_LOCKS
        mangle.MAX_LOCKS = 10
        self.store = Store(0, self.mktemp(),self.mktemp())

    def tearDown(self):
        from atop import store as mangle
        mangle.MAX_LOCKS = self.oldml

    def testBreakIt(self):
        """Allocate too many locks and try to recover.
        """
        def initx():
            nx = Item(self.store)
            nx.step = 1
            np = Pool(self.store)
            return np, nx
        p, x = self.store.transact(initx)
        def breakx():
            x.touch()
            x.step = 2
            for n in xrange(100000):
                # print 'count:', n, 'locks:', self.store.env.lock_stat()['nlocks']
                i = Item(self.store, "hello")
                p.addItem(i)
                list(p.queryIndex())
        self.assertRaises(DBNoMemoryError, self.store.transact, breakx)
        self.assertEquals(x.step, 1)
        self.assertEquals(len(self.store.transact(lambda : list(p.queryIndex()))), 0)
        gc.collect()
        self.failUnless(self.store.cache.has(tuple(x.getItemPath())))
        self.assertEquals(len(self.store.cache.data), 2)

# TortureTest.skip = "This consumes too many resources for how little it tests."
