# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

import types

from twisted.python import components

from nevow.iwoven import IGettable, ISettable, IContainer, IData
from nevow.stan import directive

def convertToData(context, obj):
    adapter = IGettable(obj, None, persist=False)
    if adapter is not None:
        obj = adapter.get(context)
    return obj


class DirectiveAccessor(components.Adapter):
    __implements__ = IGettable,

    def get(self, context):
        # The directive (self.original) is the data at depth=1
        data = context.locate(IData, depth=2)
        container = IContainer(data, None, persist=False)
        if container is None:
            raise RuntimeError, "%s does not implement IContainer, and there is no registered adapter." % data
        child = container.child(context, self.original)
        return convertToData(context, child)


class FunctionAccessor(components.Adapter):
    __implements__ = IGettable,
    def get(self, context):
        # The function (self.original) is the data at depth=1
        return self.original(context, context.locate(IData, depth=2))


class DictionaryContainer(components.Adapter):
    __implements__ = IContainer, IGettable
    
    def child(self, context, name):
        return self.original.get(name, None)

    def get(self, context):
        return dict(
            [(k, convertToData(context, v)) for (k, v) in self.original.items()]
        )


class ListContainer(components.Adapter):
    __implements__ = IContainer, IGettable

    def child(self, context, name):
        try:
            i = int(name)
        except ValueError:
            return None
        return self.original[i]

    def get(self, context):
        return [convertToData(context, x) for x in self.original]


