##

import cgi
from copy import copy

from twisted.web import server
from twisted.web import resource
from twisted.web.woven import guard
from twisted.python import components

from nevow import context
from nevow import iwoven
from nevow import freeform


class GuardInitGenerator(components.Adapter):
    def getResourceForRequest(self, request):
        name = request.postpath[-1]
        if name == guard.INIT_PERSPECTIVE or name == guard.DESTROY_PERSPECTIVE:
            ## We might need to set up a session;
            ## the session resource will delegate getChild to the
            ## guard resource if we already have a session
            print "GUARD INIT"
            sessioner = request.site.resource
            return sessioner.getChild(name, request)


class FormPostGenerator(components.Adapter):
    def getResourceForRequest(self, request):
        ## The method or property name we are going to validate against/affect
        bindingName = None

        name = request.postpath[-1]
        if name.startswith('freeform_post:'):
            bindingName = name.split(':')[-1]
        if name.startswith('freeform-action-post:'):
            request.args['freeform-actee'] = name.split(':')[-1]
            bindingName = request.args['freeform-action'][0]
        if bindingName:
            request.postpath.pop()
            if not request.postpath:
                request.postpath.append('')
            res = resource.getChildForRequest(self.original, request)

            ctx = context.WovenContext()
            ctx.remember(res, resource.IResource)
            ctx.remember(request, iwoven.IRequest)
            configurable = freeform.locateConfigurable(ctx, getattr(res, 'original', res))

            return self.postForm(request, res, configurable, ctx, bindingName, request.args)

    def postForm(self, request, res, configurable, ctx, bindingName, args):
        return configurable.postForm(ctx, bindingName, args)


class PrettyPrinter(components.Adapter):
    def getResourceForRequest(self, request):
        name = request.postpath[-1]
        if name == '$$HTMLSource':
            request.postpath.pop()
            collector = []
            oldwrite = request.write
            oldfinish = request.finish
            def interceptWrite(written):
                print "written", written
                collector.append(written)
            def interceptFinish():
                print "finish"
                source = ''.join(collector)
                from twisted.web.microdom import parseString
                from twisted.xish.domish import escapeToXml
                tree = parseString(source)
                textized = tree.toprettyxml()
                oldwrite('<html><body><pre>')
                oldwrite(escapeToXml(textized))
                oldwrite('</pre></body></html>')
                oldwrite(textized)
                oldfinish()
            request.write = interceptWrite
            request.finish = interceptFinish


class NevowRequest(server.Request):
    """A Request subclass which does additional
    processing if a form was POSTed. When a form is POSTed,
    we create a cgi.FieldStorage instance using the data posted,
    and set it as the request.fields attribute. This way, we can
    get at information about filenames and mime-types of
    files that were posted.
    """
    def process(self):
        # extra request parsing
        if self.method == 'POST':
            t = self.content.tell()
            self.content.seek(0)
            self.fields = cgi.FieldStorage(self.content, self.received_headers,
                                           environ={'REQUEST_METHOD': 'POST'})
            self.content.seek(t)
        return server.Request.process(self)


class NevowSite(server.Site):
    """A Site subclass which does additional pre-processing before
    delegating the request rendering to a Resource instance.
    By default, this Site is able to deal with Guard login/logout
    requests, Freeform form posts, and pretty-printing the HTML
    source of any page.
    """
    requestFactory = NevowRequest
    generatorFactories = [
        FormPostGenerator,
        PrettyPrinter,
    ]
    def __init__(self, *args, **kw):
        server.Site.__init__(self, logPath="web.log", *args, **kw)
        self.resourceGenerators = [
            x(self) for x in self.generatorFactories
        ]

    def getResourceFor(self, request):
        request.site = self
        request.sitepath = copy(request.prepath)
        res = None
        for gener in self.resourceGenerators:
            res = gener.getResourceForRequest(request)
            if res is not None:
                break
        if res is None:
            res = resource.getChildForRequest(self.resource, request)
        return res

