# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.


DEBUG = False


from twisted.python import components
from twisted.web import server

from weakref import ref


class IEventMaster(components.Interface):
    pass


class EventMaster(components.Adapter):
    _request = None
    def __init__(self, request):
        self._request = request
        self._subscribers = {}

    def subscribe(self, identifier, subscriber):
        """Subscribe to events sent to the given identifier.
        
        Returns a token which should be passed to unsubscribe when done.
        """
        if DEBUG:
            print "SUBSCRIBE", self, identifier, subscriber
        reference = ref(subscriber)
        self._subscribers.setdefault(identifier, []).append(reference)
        return identifier, reference

    def unsubscribe(self, token):
        """Unsubscribe the given token from events.
        """
        if DEBUG:
            print "UNSUBSCRIBE", token
        identifier, reference = token
        self._subscribers[identifier].remove(reference)

    def publish(self, identifier):
        """Notify the listeners on a given identifier that an event has occurred.
        """
        if DEBUG:
            print "PUBLISH", self, identifier,
        subscribers = self._subscribers.get(identifier, [])
        subscribers = subscribers[:]
        for reference in subscribers:
            sub = reference()
            if DEBUG:
                print sub
            if sub is None:
                subscribers.remove(reference)
            else:
                sub.model.invalidateCache()
                sub.modelChanged({'request': self._request})
        if DEBUG:
            print "done"

    def __getstate__(self):
        d = self.__dict__.copy()
        d['_subscribers'] = {}
        d['_request'] = None
        return d


components.registerAdapter(EventMaster, server.Request, IEventMaster)

