# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.


from twisted.python import components


class IRendererFactory(components.Interface):
    """A renderer factory is capable of taking a renderer directive (a string)
    and returning a callable which when called, will render a portion of DOM.
    """
    def renderer(self, context, name):
        """Given a context object and a name, return a callable which responds
        to the signature (context, data) or (data) and returns an object which
        is adaptable to ISerializable.
        """


class IData(components.Interface):
    """Any python object to be used as model data to be passed
    to view functions. Used for marking the context stack only.
    
    ANY python object is said to implement IData.
    """


class IGettable(components.Interface):
    def get(self, context):
        """Return the data
        
        Return any object
        """


class ISettable(components.Interface):
    def set(self, context, data):
        """Set the data

        This might be removed.
        """


class IContainer(components.Interface):
    def child(self, context, name):
        """Return a conceptual child; an attribute, or a key,
        or the result of a function call.
    
        Returns any object; the result may be adapted to IGettable
        if possible.
        
        Return None if the adaptee does not have a child with the
        given name.
        
        TODO: Maybe returning None is bad, and .child should just
        raise whatever exception is natural
        """


class IRequest(components.Interface):
    """A web request
    """


class ISerializable(components.Interface):
    def serialize(self, context, stream):
        """Serialize the adaptee to the given stream, with the given context
        stack if necessary.
        """


class IHand(components.Interface):
    """A marker interface which indicates what object the user is currently
    holding in their hand. This is conceptually the "result" of the last operation;
    this interface can be used to mark a status string which indicates whether
    the most recent operation completed successfully, or can be used to hold
    an object which resulted from the most recent operation.
    """

