# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

from nevow.iwoven import ISerializable
from nevow.stan import Tag, xml, directive

from twisted.python import components

class MicroDomTextSerializer(components.Adapter):
    __implements__ = ISerializable,
    
    def serialize(self, context, stream):
        if self.original.raw:
            yield self.original.nodeValue
        else:
            from twisted.xish.domish import escapeToXml
            yield escapeToXml(self.original.nodeValue)


class MicroDomCommentSerializer(components.Adapter):
    __implements__ = ISerializable,
    
    def serialize(self, context, stream):
        yield xml("<!--%s-->" % self.original.data)

class MicroDomEntityReferenceSerializer(components.Adapter):
    __implements__ = ISerializable,

    def serialize(self, context, stream):
        yield xml(self.original.nodeValue)


class MicroDomElementSerializer(components.Adapter):
    __implements__ = ISerializable,

    directiveMapping = {
        'render': 'renderer',
        'data': 'data',
        'observer': 'observer'
    }
    attributeList = [
        'pattern', 'slot', 'macro', 'fill-slot', 'key',
    ]

    def serialize(self, context, stream):
        element = self.original
        attrs = dict(element.attributes) # get rid of CaseInsensitiveDict
        specials = {}
        attributes = self.attributeList
        directives = self.directiveMapping
        for k, v in attrs.items():
            # I know, this is totally not the way to do xml namespaces but who cares right now
            ## I'll fix it later
            if not k.startswith('nevow:'):
                continue
            _, nons = k.split(':')
            if nons in directives:
                ## clean this up by making the names more consistent
                specials[directives[nons]] = directive(v)
                del attrs[k]
            if nons in attributes:
                specials[nons] = v
                del attrs[k]

        yield ISerializable(
            Tag(
                element.tagName,
                attributes=attrs,
                children=element.childNodes,
                specials=specials
            )
        ).serialize(context, stream)


