# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.


from twisted.trial import unittest

from nevow import stan


class TestProto(unittest.TestCase):
    def test_proto(self):
        tagName = "hello"
        proto = stan.Proto(tagName)
        self.assertEquals(tagName, str(proto))

    def test_callCreatesTag(self):
        proto = stan.Proto("hello")
        tag = proto(world="1")
        self.assertEquals(proto, tag.tagName)
        self.assertEquals(tag.attributes['world'], '1')

    def test_getItemCreatesTag(self):
        proto = stan.Proto("hello")
        tag = proto[proto]
        self.assertEquals(proto, tag.tagName)
        self.assertEquals(tag.children, [proto])


proto = stan.Proto("hello")


class TestTag(unittest.TestCase):
    def test_clone(self):
        tag = proto(hello="world")["How are you"]
        clone = tag.clone()
        self.assertEquals(clone.attributes['hello'], 'world')
        self.assertNotIdentical(clone.attributes, tag.attributes)
        self.assertEquals(clone.children, ["How are you"])
        self.assertNotIdentical(clone.children, tag.children)

    def test_clear(self):
        tag = proto["these are", "children", "cool"]
        tag.clear()
        self.assertEquals(tag.children, [])

    def test_specials(self):
        tag = proto(data=1, renderer=str, observer="1", pattern="item", slot="aSlot", macro="aMacro", **{'fill-slot': "someSlot"})
        self.assertEquals(tag.data, 1)
        self.assertEquals(tag.renderer, str)
        self.assertEquals(tag.observer, "1")
        self.assertEquals(tag.pattern, "item")
        self.assertEquals(tag.slot, "aSlot")
        self.assertEquals(tag.macro, "aMacro")
        self.assertEquals(getattr(tag, 'fill-slot'), "someSlot")

