#!/usr/bin/env python2.3
import random, sys, os, time
import tempfile
import shutil
from os.path import join as opj

HACKPATH = os.path.abspath(opj(__file__, "..", "..", ".."))
if HACKPATH not in sys.path:
    sys.path.append(HACKPATH)

import os
import cPickle as pickle
from pychartdir.pychartdir import *
from twisted.python.reflect import namedAny

class Graph:
    CHART_DIM = (800, 550)
    PLOT_AREA = (60, 30, 720, 470)

    DATA_COLOR = 0x000080
    TREND_COLOR = 0x008000

    def __init__(self, name):
        self.name = name

    def render(self, stats, percent=False, linear=False, style='StepLine'):
        label = self.name
        if percent:
            label = label + ' %'

        if linear:
            stats = stats[-100:]

        time = [chartTime2(t) for (t, d) in stats]
        data = [d for (t, d) in stats]

        # A tweak to make sure there is a little room at the
        # right of the graph.

        time.append(time[-1] + 1000)

        # Set chart area
        c = XYChart(*self.CHART_DIM)

        # Set the plot area
        c.setPlotArea(*self.PLOT_AREA)

        # Add a chart layer using the given data
        layer = getattr(c, 'add%sLayer' % (style,))(data, self.DATA_COLOR, label)

        if not linear:
            layer.setXData(time)
            c.xAxis().setDateScale3()

        c.xAxis().setMargin(0, 0)

        ds = layer.getDataSet(0)
        ds.setDataSymbol(DiamondSymbol, 8)

        trendLayer = c.addTrendLayer(data, self.TREND_COLOR, "Trend Line")
        if not linear:
            trendLayer.setXData(time)

        c.addTitle(label)

        if percent:
            c.yAxis().setLinearScale(0, 100)
        else:
            c.yAxis().setAutoScale(0.15, 0, 1)

        fd, name = tempfile.mkstemp()
        os.close(fd)
        c.makeChart(name)
        return name

class RecentCommitsGraph(Graph):
    def render(self, stats, percent=False, linear=False):
        return Graph.render(self, stats[-30:], percent=percent, linear=linear, style='Bar')

class VersusCommitsGraph(Graph):
    def render(self, stats, percent=False):
        return Graph.render(self, stats, percent=percent, linear=True, style='StepLine')

def main(statsFile, graphDir, historyFile):
    """
    @param statsFile: Path to file containing pickled benchmark statistics.
    @param graphDir: Path to directory into which to place graph files.
    @param historyFile: Name of the file into which historical stats are written.
    """
    graphDir = os.path.abspath(graphDir)
    history = os.path.join(graphDir, historyFile)

    data = {}
    if os.path.exists(history):
        data = pickle.load(file(history))

    if statsFile:
        newtimestamp = os.path.getmtime(statsFile)
        newstats = pickle.load(file(statsFile))

        for benchName, stats in newstats.iteritems():
            for statName, (val, scale) in stats.iteritems():
                print 'Updating history for', benchName + '-' + statName
                l = data.setdefault("%s-%s" % (benchName, statName), (scale, []))
                l[1].append((newtimestamp, val))

        pickle.dump(data, open(history, 'wb'))

    for name, (scale, stats) in data.iteritems():
        print 'Generating graphs for', name
        # Oy! This is getting hackier by the day :(
        mName, tName = name.split('-', 1)
        method = namedAny(mName)
        percent = getattr(method, 'type', None) == 'percent'
        name = getattr(method, 'name', scale)
        for (gName, gType) in GRAPHS:
            odir = opj(graphDir, gName)
            ofile = opj(odir, os.extsep.join((name, 'png')))
            if not os.path.isdir(odir):
                os.mkdir(odir)

            g = gType(name).render(stats, percent=percent)
            shutil.move(g, ofile)
            os.chmod(ofile, 0644)

GRAPHS = [
    ('', Graph),
    ('RecentCommits', RecentCommitsGraph),
    ('VersusCommit', VersusCommitsGraph)]

if __name__ == '__main__':
    main(sys.argv[1], sys.argv[2], "stats.history")
