# -*- test-case-name: quotient.test.test_extract -*-

# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

import re

from atop import store
from atop import tpython
from atop.powerup import Powerup

from nevow import tags

from quotient import cleanup
from quotient import provisioner
from quotient import mimemessage
from quotient import taglib
from quotient.web.root import Base
from quotient.smarts.note import INotebook, Note

from twisted.python.components import Interface

class ICommand(Interface):
    pass

class Commander(Powerup):
    dependencies = []
    invisible = True
    totalExtracted = 0
    powerupName = "Commander"
    _implements__ = ICommand,

    def setUpPools(self, avatar, name=None):
        avatar.setComponent(ICommand, self)
        commandsPool = store.Pool(avatar, "Commands")
        self.commandsPoolRef = commandsPool.referenceTo()
        avatar.getRootPool().addItem(commandsPool)

    def commandsPool(self):
        return self.commandsPoolRef.getItem()

    def commandRaw(self, request, cmdStr):
        order = Note(self.store, cmdStr)
        self.commandsPool().addItem(order)
        # the parsing was broken, so I just turned it off
        cmd, arg = cmdStr, ''
        return self.command(request, cmd.strip().lower(), arg.strip())

    def command(self, request, cmd, arg):
        if cmd == 'j':
            # jump to pool
            pass
        if cmd == 'p':
            # jump to pup
            pass
        # the return value from here will show up to the user
        return "I didn't know what to do with your command: %r %r" % (cmd, arg)


class CommanderDisplay(Base):
    def render(self, context, data):
        return self.document

    document = taglib.page(title="Commander")[
        tags.p["Hello"],
    ]
