# -*- test-case-name: quotient.test.test_garbage -*-
# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

"""
Garbage collector!
"""

from atop import fwd
from atop import sched
from atop import store
from atop import powerup

from nevow import formless
from nevow import freeform

from twisted.python import components

from quotient import cleanup
from quotient import mimemessage
from quotient.stats import formatSize

DAY = 60 * 60 * 24

class ICollector(components.Interface):
    pass

class GarbageCollector(powerup.Powerup, sched.Schedulable):

    __implements__ = (powerup.Powerup.__implements__, sched.Schedulable.__implements__, ICollector)

    dependencies = [sched.SubScheduler, cleanup.Cleanup]

    powerupName = "Trash Man"

    interval = DAY
    age = 7

    def setUpPools(self, avatar):
        trash = store.Pool(avatar, name="Trash")
        trash.addIndex(store.NameIndex())
        trash.addIndex(store.DateIndex())
        trash.addIndex(mimemessage.SizeIndex())
        trashSpam = store.Pool(avatar, name="Spam")
        trashSpam.addIndex(store.DateIndex())
        trashSpam.addIndex(mimemessage.SizeIndex())
        trash.addItem(trashSpam)
        avatar.getRootPool().addItem(trash)
        self.trashRef = trash.referenceTo()
        self.trashSpamRef = trashSpam.referenceTo()
        self.spamRef = cleanup.IClean(avatar).spamRef
        avatar.setComponent(ICollector, self)
        self.reset(self.interval)

    def _poolSize(self, pool):
        q = pool.queryIndex('size', result=store.INDEX)
        return sum(q)

    def run(self):
        self.reset(self.interval)
        self.collect()

    def collect(self):
        keep = not cleanup.ISpambayes(self.store).incinerateSpam
        now = sched.IScheduler(self.store).clock() - (self.age * DAY)
        then = now
        t = self.trashSpamRef.getItem()
        s = self.spamRef.getItem()
        def _():
            for i in s.queryIndex(store.RECEIVED, startKey=then, backwards=True):
                s.removeItem(i)
                if keep:
                    t.addItem(i)
        self.store.transact(_)

    def empty(self, all=False):
        self.reset(self.interval)
        dr = None
        if all:
            t = self.trashRef.getItem()
            dr = self.trashSpamRef.getItem()
        else:
            t = self.trashSpamRef.getItem()
        def _():
            c = 0
            s = self._poolSize(t)
            for i in t.queryIndex():
                if i is not dr:
                    # XXX
                    # Delete from database
                    # Delete from disk
                    c += 1
                    t.removeItem(i)
            return (c, s)
        return self.store.transact(_)


from quotient.web import root
from quotient.taglib import page
from quotient.prefup import touchNForward

class IGarbageCollector(formless.TypedInterface):
    age = formless.Integer(label="Collection Threshold",
                           description="Age of messages to move from Spam to Trash, in days.",
                           default="7")

    def emptySpam(self):
        """Delete all spam from the trash pool."""
    emptySpam = formless.autocallable(emptySpam)

    def emptyAll(self):
        """Delete all messages from the trash pool."""
    emptyAll = formless.autocallable(emptyAll)

class GarbageDisplay(root.Base, freeform.Configurable):
    __implements__ = (IGarbageCollector,)

    document = page(title="Trash Manager Configuration")[ freeform.configure ]

    def emptySpam(self):
        c, s = self.original.empty(all=False)
        return "Deleted %d spam message%s totalling %s." % (c, plural(c), formatSize(s))

    def emptyAll(self):
        c, s = self.original.empty(all=True)
        return "Deleted %d trash message %s totalling %s." % (c, plural(c), formatSize(s))

    age = touchNForward("age")

    # XXX TODO Something about getting rid of the necessity for this
    postLocation = property(lambda self: self.original.storeID)
