# -*- test-case-name: quotient.test.test_prefup -*-

# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

from atop.powerup import Powerup
from atop.store import Pool, NameIndex

from nevow import formless

from twisted.python.components import Interface

from quotient import taglib
from quotient import timezones
from quotient.web.root import Base


def touchNForward(propName):
    def get(self):
        return getattr(self.original, propName)

    def set(self, val):
        self.original.touch()
        setattr(self.original, propName, val)

    return property(get, set)

def stringifyTimezone(tz):
    ## I have no freaking clue how to do this
    return str(tz)

class PreferenceGrouping(formless.TypedInterface):
    """All these preferences can be edited and submitted with one form.
    """
    realname = formless.String(
        label = "Real name",
        description = "Your real name.")
    replyto = formless.String(
        label="Default From:",
        description="The default email address to use as the From address when composing new messages.")
    timezone = formless.Choice(
        timezones.zones,
        label="Timezone:",
        description="The timezone to display all times in.",
        stringify=stringifyTimezone)
    emergencyContact = formless.String(
        label="Non-Quotient Address:",
        description="An address to which correspodence about lost passwords can be directed.")
    itemsPerPage = formless.Choice([5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 100],
                                   label="Items Per Page",
                                   description="The maximum number of items to show on a single page.")
    showInvisible = formless.Boolean(
        label = "Enable Developer Mode",
        description = 'Whether to show "System" items when browsing.'
        )

class IUserPreferences(formless.TypedInterface):
    PreferenceGrouping = PreferenceGrouping

    def setPref(self, pref=formless.String(), value=formless.String()):
        pass
    setPref = formless.autocallable(setPref, invisible=True)


class PrefsDisplay(Base):
    __implements__ = IUserPreferences,
    document = taglib.configurePage(title="Personal Preferences")
    postLocation = property(lambda self: self.original.storeID)

    realname = touchNForward('realname')
    emergencyContact = touchNForward("emergencyContact")
    replyto = touchNForward('replyto')
    timezone = touchNForward('timezone')
    itemsPerPage = touchNForward('itemsPerPage')
    showInvisible = touchNForward('showInvisible')

    def setPref(self, pref, value):
        self.original.setPref(pref, value)

    
class IPrefs(Interface):
    def setPref(self, pref, value):
        """Associate the given value with the given preference.

        @type pref: C{str}
        """

    def getPref(self, pref, default=None):
        "return a preference with the name 'pref'. Return and set 'default' if this pref hasn't been set before."


class Prefup(Powerup):
    __implements__ = IPrefs, Powerup.__implements__
    powerupName = 'Personal Preferences'

    showInvisible = False

    def setUpPools(self, avatar):
        avatar.setComponent(IPrefs, self)
        self.setDefaults()

    def setMe(self, person):
        self.meRef = person.referenceTo()
        self.touch()

    def getMe(self):
        return self.meRef.getItem()

    def setDefaults(self):
        self.meRef = None                 # Who is the Person that own items I create
        self.realname = ''
        self.timezone = timezones.UTC
        self.replyto = ''
        self.itemsPerPage = 20
        self.emergencyContact = ''
        self.whitelistOnSend = True    # Auto-add people in sent pool to WL
        self.whitelistOnly = False     # Only let mail through if sender on WL
        self.location = ''             # location code like zip or post code
        self.autoHTMLRender = False    # Render full HTML if person is in WL or SB is sure it's clean
        self.cliPrompt = 'CLI:'
        vis = Pool(self.store, 'Visible Items')
        vis.addIndex(NameIndex())
        vis.suggestIndex('name', backwards=False)
        self.visiblePoolRef = vis.getRef()
        self.store.getRootPool().addItem(vis)
        pri = Pool(self.store, 'Primary Navigation')
        pri.addIndex(NameIndex())
        pri.suggestIndex('name', backwards=False)
        self.primaryNavPoolRef = pri.getRef()
        self.store.getRootPool().addItem(pri)
        
    def setPref(self, pref, value):
        self.touch()
        setattr(self, pref, value)

    def getPref(self, pref, default=None):
        if pref[0] == '_':
            raise AttributeError, pref
        if not hasattr(self, pref):
            setattr(self, pref, default)
        return getattr(self, pref)

    def getVisiblePool(self):
        return self.visiblePoolRef.getItem()

    def getPrimaryNavPool(self):
        return self.primaryNavPoolRef.getItem()

    def makeVisible(self, item):
        if item not in self.getVisiblePool():
            self.getVisiblePool().addItem(item)

    def addToPrimaryNav(self, item):
        if item not in self.getPrimaryNavPool():
            self.getPrimaryNavPool().addItem(item)
        

