# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

import re

from atop import store
from atop import powerup

from twisted.python import components

from quotient import mimemessage
from quotient.smarts import base
from quotient.prefup import IPrefs

class Filer(base.ExtractCollectorBase):
    pass

def matchMailman(item, listExpr = re.compile(r'''<(?P<listId>[^>]*)>''')):
    listId = item['List-Id']
    if listId:
        m = listExpr.search(listId)
        if m:
            return m.group('listId')
    return None

def matchMajorDomo(item):
    listId = item['X-Mailing-List']
    if listId:
        return listId.replace('@', '.')
    return None

class MailingListFiler(Filer):

    matchers = [matchMailman, matchMajorDomo]

    def setUpPools(self, avatar):
        Filer.setUpPools(self, avatar, "Mailing Lists", None, False)
        self.extractPool().addIndex(store.NameIndex())
        avatar.getRootPool().addItem(self.extractPool())
        IPrefs(avatar).addToPrimaryNav(self.extractPool())

    def extractItem(self, item):
        listName = self.listNameFromItem(item)
        if listName:
            lists = self.extractPool()
            listPool = lists.getIndexItem('name', listName, None)
            if listPool is None:
                listPool = store.Pool(self.store, name=listName)
                listPool.addIndex(mimemessage.FlagsIndex())
                lists.addItem(listPool)
            listPool.addItem(item)
            return base.FILED
        return None

    def listNameFromItem(self, item):
        for f in self.matchers:
            n = f(item)
            if n is not None:
                return n
        return None

class IRouteTarget(components.Interface):
    def receive(self, route, item):
        pass

class Router(Filer):
    greyedOut = True
    invisible = True

    def setUpPools(self, avatar):
        Filer.setUpPools(self, avatar, "Recipient Routing", None, False)

    def extractItem(self, item):
        route = getattr(item, "route", None)
        if route:
            return self.deliverRouted(route, item)
        return None

    def deliverRouted(self, route, item):
        powerupName = route.pop()
        p = powerup.IPowerStation(self.store).getPowerupsPool()
        for pup in p.queryIndex():
            if pup.powerupName.lower() == powerupName.lower():
                deliver = IRouteTarget(pup, default=None)
                if deliver is not None:
                    if deliver.receive(route, item):
                        return base.FILED
        return None
