# -*- test-case-name: quotient.test.test_extract -*-

# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

from atop.store import Item, Pool
from atop.powerup import Powerup, IPowerup
from atop.fwd import ItemForwardingComponent

from twisted.python.components import Interface

from quotient.web.root import Base, TabularBase, column
from nevow import formless
from nevow import freeform
from nevow import tags
from quotient import taglib
from quotient.searchup import Searchup, ISearch
from quotient.prefup import IPrefs


class INote(Interface):
    pass

class IEditableNote(formless.TypedInterface):
    def editNote(
        self,
        name=formless.Text(),
    ):
        """
        """
    editNote = formless.autocallable(editNote)
    
class Note(Item):
    __implements__ = INote, IEditableNote
    def __init__(self, store, text):
        Item.__init__(self, store, name=text)
        self.description = text

    def editNote(self, name):
        def _():
            self.description = self.name = name
            self.touch()
            self.change()
        self.store.transact(_)
        
    def __repr__(self):
        return 'Note: %s' % self.description

class NoteDisplay(Base):
    __implements__ = (Base.__implements__, INote)
    document = taglib.page(title="Note")[
        freeform.configure
    ]


class INotebook(Interface):
    pass
    
class Notebook(Powerup):
    __implements__ = (INotebook, IPowerup, )
    dependencies = [Searchup, ]

    powerupName = "Notes"

    def setUpPools(self, store):
        store.setComponent(INotebook, self)
        p = Pool(store, name='Notes')
        self.notesPoolRef = p.referenceTo()
        store.getRootPool().addItem(p)
        srch = ISearch(store)
        srch.indexPool(p)
        self.invisible  = True
        
    def notesPool(self):
        return self.notesPoolRef.getItem()

    def command(self, cmd, arg):
        if 'note'.startswith(cmd):
            def _():
                t = Todo(self.store, arg)
                self.notesPool().addItem(t)
            self.store.transact(_)

class ITodo(Interface):
    pass

class IEditableTodo(formless.TypedInterface):
    def editTodo(
        self,
        done=formless.Boolean(),
        name=formless.Text(),
    ):
        """
        """
    editTodo = formless.autocallable(editTodo)

class TodoRow(TabularBase):
    def render_done(self, context, data):
        todo = ITodoList(data.store)
        if data.done:
            return tags.a( href="/me/%s?undo=%s" % (todo.storeID, data.storeID)) [ "Mark as undone" ]
        else:
            return tags.a( href="/me/%s?done=%s" % (todo.storeID, data.storeID)) [ "Mark as done" ]
    column(render_done, True, 250)

class Todo(Item):
    __implements__ = ITodo, INote, IEditableTodo
    
    def __init__(self, store, text, done=False):
        Item.__init__(self, store, name=text)
        self.done = done
        self.description = text

    def setDone(self, done=True):
        def _():
            self.done = done
        self.store.transact(_)

    def editTodo(self, name, done):
        def _():
            self.name = name
            self.description = name
            self.done = done
            self.touch()
        self.store.transact(_)

    def __repr__(self):
        if self.done:
            return 'Todo: %s (Done)' % self.description
        else:
            return 'Todo: %s (Not Done)' % self.description

class ITodoList(Interface):
    pass

class TodoList(Powerup):
    __implements__ = (ITodoList, IPowerup)
    dependencies = [Searchup, ]

    powerupName = "Todo Manager"

    def setUpPools(self, store):
        store.setComponent(ITodoList, self)
        p = Pool(store, name='Todo')
        self.todoPoolRef = p.referenceTo()
        store.getRootPool().addItem(p)
        srch = ISearch(store)
        srch.indexPool(p)
        self.invisible  = True

    def todoPool(self):
        return self.todoPoolRef.getItem()
    
    def command(self, cmd, arg):
        if 'todo'.startswith(cmd):
            def _():
                t = Todo(self.store, arg)
                self.todoPool().addItem(t)
            self.store.transact(_)

class TodoListDisplay(Base):
    __implements__ = ITodoList
    
    document = taglib.page(title="Todo List")[
        tags.p["Todo List"],
    ]
    
    def render(self, request):
        if request.args.has_key('done'):
            todoID = request.args['done'][0]
            done = True
        elif request.args.has_key('undo'):
            todoID = request.args['undo'][0]
            done = False
        else:
            return Base.render(self, request)
        todo = self.original.store.getItemByID(int(todoID))
        todo.setDone(done)
        request.redirect(request.getHeader('referer'))
        return Base.render(self, request)

    
    
