# -*- test-case-name: quotient.test.test_tdb -*-

# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.


from twisted.python import components
from twisted.web.woven import guard
from twisted.web import resource


from nevow import tags
from nevow import freeform
from nevow import iwoven

from twisted.python.components import Interface, CannotAdapt

from quotient.web import webinterfaces
from quotient.copyright import version

from atop import credup


class ITimer(Interface):
    """Marker interface to represent the time this page started rendering.
    """


import time
def startTimer(context, data):
    context.remember(time.time(), ITimer)
    return ''


def showTimer(context, data):
    now = time.time()
    then = context.locate(ITimer)
    return tags.span(_class="timer")["%0.2f ms" % ((now - then) * 1000)]


def linkToAvatarAspect(aspect, label):
    def getAspect(context, data):
        ava = context.locate(resource.IResource).avatar
        return tags.a(href=aspect(ava).storeID)[
            label
        ]
    return getAspect


def linkToPool(poolName, label=None):
    def getPool(context, data):
        ava = context.locate(resource.IResource).avatar
        rootPool = ava.rootPoolRef.getItem()
        try:
            pool = rootPool.getIndexItem('name', poolName, None)
            if pool is None:
                return "The pool %s was not found" % poolName
            return tags.a(href='/me/%s' % str(pool.storeID)) [
                label or poolName
            ]
        except AttributeError:
            return "There was a problem locating the pool %s." % poolName

    return getPool

def linkToPowerup(powerupName, label=None):
    def getPowerup(context, data):
        ava = context.locate(resource.IResource).avatar
        rootPool = ava.rootPoolRef.getItem()
        powerups = rootPool.getIndexItem('name', 'powerups')
        it = powerups.getIndexItem('name', powerupName, None)
        if it is None:
            return "The powerup %s was not found" % powerupName
        return tags.a(href=str(it.storeID))[
            label or powerupName
        ]
    return getPowerup
    

def createSearchForm(context, data):
    from quotient import searchup
    ava = context.locate(resource.IResource).avatar
    try:
        srchup = searchup.ISearch(ava)
    except CannotAdapt:
        return tags.span(_class="disabled")['Search not installed.']
    
    return tags.form(action=str(srchup.storeID))[
        tags.input(type="text", name="query", style="width: 125px"),
        tags.input(type="submit")
    ]


def getAttribute(name):
    return lambda context, data: getattr(data, name, "No attribute named %r found on %r." % (name, data))


def eitherOr(eitherer, orer):
    return lambda context, data: (
        getattr(data, eitherer, None) or 
        getattr(data, orer, None) or 
        "No attribute named %r or %r found on %r." % (eitherer, orer, data))


name = getAttribute('name')
anId = getAttribute('storeID')
nameOrId = eitherOr('name', 'storeID')


def containerList(context, data):
    if not getattr(data, 'pools', None) or isinstance(data, credup.Avatar):
        return ''
    return tags.ul(_class="container-list")[
        "Back to: ", [
            tags.li(_class="backreference", data=it)[
                tags.a(href=anId)[nameOrId]
            ]
            for it in data.iterpools()
        ]
    ]


def commandLineAction(context, data):
    try:
        from quotient import commander
        commander.ICommand(getattr(data, 'store', None))
    except components.CannotAdapt:
        return ''
    req = context.locate(iwoven.IRequest)
    for slot in context._locateSlots('prompt'):
        slot["Command:"]
    pth = req.path.split('/')
    if req.path[-1] != '/' and pth[-1] != 'quotient_command_line':
        act = action="%s/quotient_command_line" % pth[-1]
    else:
        act = action="quotient_command_line"
    for form in context._locateKeys('form'):
        form(action=act)
    result = req.getComponent(iwoven.IHand, None)
    if result:
        context.slotted('result')[ result ]
    return context.tag


def visiblePowerups(context, data):
    from quotient import prefup
    ava = context.locate(resource.IResource).avatar
    req = context.locate(iwoven.IRequest)
    try:
        cur = int(req.prepath[-1])
    except ValueError:
        cur = ''
    try:
        return [
            tags.li(
                _class=x.storeID == cur and "tab-highlighted" or "tab-normal"
            )[
                tags.a(href=x.storeID)[
                    x.name
                ]
            ]
            for x in list(prefup.IPrefs(ava).getPrimaryNavPool().queryIndex())
        ]
    except components.CannotAdapt:
        return ''


def navigation(context, data):
    from quotient import prefup
    req = context.locate(iwoven.IRequest)
    ava = context.locate(resource.IResource).avatar

    dash = prefup.IPrefs(ava).getVisiblePool()
    try:
        dashid = int(req.prepath[-1])
    except ValueError:
        dashid = ''
    return tags.ul(_class="navigation")[
        tags.li(_class=req.path == '/me/' and "tab-highlighted" or "tab-normal")[tags.a(href='/me/')["Home"]],
        tags.li(_class=req.path == '/me/compose' and "tab-highlighted" or "tab-normal")[tags.a(href='/me/compose')["Compose"]],
        visiblePowerups,
        tags.li(_class=dashid == dash.storeID and 'tab-highlighted' or 'tab-normal')[tags.a(href="/me/%s" % dash.storeID)["Dashboard"]],
        containerList,
        tags.li[tags.a(_class="logout", href="/"+guard.DESTROY_PERSPECTIVE)["Logout"]],
        tags.li(_class="version")['Quotient v.', version]
    ]


class page:
    def __init__(self, title, anonymous=False, **kw):
        self.b = body = tags.invisible()

        if not anonymous:
## TODO Make these links aware of an application not installed on /
            nav = navigation
        else:
            nav = ''

        self.html = [
            tags.xml('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">\n'),
            tags.html(xmlns="http://www.w3.org/1999/xhtml", lang="en", **{'xml:lang': "en"})[
        startTimer,
        tags.head[
            tags.title[ title ],
            tags.link(rel="stylesheet", href="/css"),
            tags.link(rel="stylesheet", href="/freeform_css"),
        ],
        tags.body[
            tags.div(_class="overall", style="width: 100%")[
                tags.div(_class="centercolumn")[
                    tags.form(renderer=commandLineAction, key="form")[
                        tags.table(style="width: 100%")[tags.tr[tags.td(width="1%", slot="prompt"), tags.td(width="99%")[
                            tags.input(type="text", name="input", style="width: 100%")
                        ]]],
                        tags.invisible(slot="result")
                    ],
                    tags.div[
                        body
                    ],
                    showTimer
                ],
                tags.div(_class="leftcolumn") [
#                    tags.img(src="/images/logos/newdivmod.png", style="width: 125px", alt="Divmod Logo"),
                    tags.img(src="/images/logos/water1.png", style="width: 200px", alt="Divmod Logo"),
                    nav,
                ]
            ]
        ]
    ]
]

    def __getitem__(self,item):
        self.b[item]
        return self.html

foo =                 tags.table(renderer=commandLineAction, border="0", style="width:100%")[
                    tags.tr(valign="top")[
                        tags.td(style="width: 125px")[tags.xml('&nbsp')],
                        tags.td(slot="prompt"),
                        tags.td(width="*")[
                            tags.form(key="form")[
                                tags.input(type="text", name="input", style="width: auto; float: right;")
                            ]
                        ]
                    ]
                ]

def messageHeader(x):
    def getIt(data):
        return headerToUTF8(data[x] or ('<No %s>' % x))
    return getIt


def headers(*names):
    def _(context, data):
        t = tags.table(style="clear: both", width="100%")
        for o in names:
            if callable(o):
                name = o.name
                dn = True
                F = o
            else:
                name = o
                dn = data[name]
                F = messageHeader(name)
            if dn:
                t[tags.tr[tags.td(_class="header-name")[name, ": "],
                          tags.td(_class="header-value")[F]]]
        return t
    return _


def br(context, data):
    return tags.div(style="clear: both")[' ']


import sre
import sys

def headerToUTF8(s):
    # Man what the heck
    found = sre.findall(r'=\?[^?]+\?[QqBb]\?[^?]+\?=', s)
    if not found:
        return unicode(s, sys.getdefaultencoding(), "replace").encode("utf8")
    for item in found:
        _, charset, encoding, text, _ = item.split('?')
        if encoding in 'Qq':
            text = text.replace("_", "=20").decode('quopri')
        else:
            text = text.decode('base64')
        try:
            s = s.replace(item, text.decode(charset))
        except LookupError:
            return '<Unknown Encoding: %s>' % charset
    return s.encode('utf8')


## from nevow.renderer import render
## from nevow.context import WovenContext
## from nevow.iwoven import ISerializable


def configurePage(*args, **kw):
    from nevow import freeform
    return page(*args, **kw)[ freeform.configure ]


__all__ = ['page', 'configurePage', 'br', 'headers', 'messageHeader', 'headerToUTF8']
