
# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

import os
import mailbox

from os.path import join as opj, isfile, abspath

from twisted.cred import portal
from twisted.cred.checkers import AllowAnonymousAccess, ICredentialsChecker
from twisted.cred.credentials import CramMD5Credentials
from twisted.web import server
from twisted.web.woven import guard
from twisted.application import service, strports

from twisted.python import usage, log

from atop.store import Store, StorageService
from atop.sched import SubScheduler, SchedulerService
from atop.powerup import IPowerStation
from atop import regadapt
from atop.credup import Avatar, StoreRealm, FwdRealm, FwdChecker

from quotient.commander import Commander
from quotient.signup import BoxOffice
from quotient.sysup import Sysup, makeUser
from quotient.proto.imapup import IMAPAccountUp
from quotient.proto.imapup import IMAP4Factory
from quotient.proto.popup import POP3Factory
from quotient.proto import sip
from quotient.proto.smtpin import ESMTPInFactory
from quotient.proto.smtpout import OutgoingMailup
from quotient.mimemessage import EmailPowerup, IMIMEDelivery
from quotient.grabbers import GrabberPowerup, IGrabberAdder
from quotient.searchup import Searchup, ISearch
from quotient.cleanup import Cleanup, Ruler, Spambayesup
from quotient.prefup import Prefup
from quotient.voiceup import Voiceup
from quotient.web import utility
from quotient.web.webup import WebPowerup
from quotient.proto.popup import Popup
from quotient.gcup import GarbageCollector
#from quotient.tasks import Taskmaster
from quotient.signup import TicketIssuer
from quotient.provisioner import Provisioner, oneYearService, IProvisioner, provisionRelation
from quotient.smarts.question import QuestionCollector
from quotient.smarts.phone import PhoneNumberCollector
from quotient.smarts.url import URLCollector
from quotient.smarts.image import ImageCollector
from quotient.smarts.person import PersonCollector
from quotient.smarts.attachment import AttachmentCollector
from quotient.smarts.list import MailingListFiler
from quotient.smarts.list import Router
from quotient.blogup import BlogUp
from quotient.sysup import Adminup, ControlPanel
from quotient.addressbook import AddressBook
from quotient.smarts.note import Notebook, TodoList
from quotient.smartenup import Smartenup
from quotient.backup import Backup
from quotient.meterer import Meterup, MeterStoreup
from quotient.web import root

from nevow import appserver


class Options(usage.Options):
    optParameters = [['webport','w', '8485','Port to run web server on.'],
                     ['pop3port','p', '10110', 'Port to run POP3 server on.'],
                     ['smtpport', 's', '10025', 'Port to run SMTP server on.'],
                     ['imap4port', 'i', '10143', 'Port to run IMAP server on.'],
                     ['datadir','d', 'QuotientService/db', 'Directory name to put database in.'],
                     ['sipport','S', '5060', "Port to run a SIP registrar service on."],
                     ['asteriskport','',None, "Port of local Asterisk server. (internal use only)"],
                     ['msgcal', 'm', 'QuotientService/user', 'Directory to put original email messages in.'],
                     ['dir', '', None, 'Import a directory full of mail'],
                     ['domain', '', 'localhost', 'Domain to run services on.'],
                     ['pop3g', 'P', None, 'Grab mail from a POP3 account. user:pass:host:port'],
                     ['imap4g', 'I', None, 'Grab mail from an IMAP4 account. user:pass:host:port'],
                     ['storeID', 'I', None, 'The ID to assign to this store.'],
                     ]

    optFlags = [['master-store', None, 'Act as a master store.']
                ]

    def __init__(self, *a, **b):
        usage.Options.__init__(self,*a,**b)
        self.users = []                 # [(user, password, dirs, mboxes)]

    def opt_user(self, user):
        """a username:password combination to create"""
        u, p = user.split(":")
        self.users.append((u,p,[],[]))

    def opt_dir(self, directory):
        if not self.users:
            raise usage.UsageError("Specify --user before --dir")
        self.users[-1][2].append(directory)

    def opt_mbox(self, mbox):
        if not self.users:
            raise usage.UsageError("Specify --user before --mbox")
        self.users[-1][3].append(mbox)

    def postOptions(self):
        if self['master-store'] and self['storeID'] is not None:
            raise usage.UsageError("You cannot specify both --master-store and --storeID")

        if self['master-store'] and self['storeID'] not in ('0', None):
            raise usage.UsageError("If you specify --master-store, you must specify --storeID=0")


        if self['master-store']:
            self['storeID'] = 0
        else:
            if self['storeID'] is None:
                self['storeID'] = 0
            else:
                try:
                    self['storeID'] = int(self['storeID'])
                except ValueError, e:
                    raise usage.UsageError("--storeID must be an integer.")

class foo:
    pass

ALL = (
    Prefup, GrabberPowerup, EmailPowerup, SubScheduler,
    GarbageCollector, MailingListFiler, Searchup,
    Popup, IMAPAccountUp, WebPowerup, Voiceup, OutgoingMailup,
    TodoList, Notebook, PersonCollector, TicketIssuer,
    QuestionCollector, PhoneNumberCollector, BlogUp, URLCollector,
    AttachmentCollector, AddressBook, Smartenup, Meterup,
    ImageCollector, Spambayesup, Ruler, Commander, Router
    )

VISIBLE = (
    Prefup, GrabberPowerup, BlogUp, Meterup,
    PersonCollector, SubScheduler, TicketIssuer,
    Ruler, GarbageCollector, Spambayesup
    )

POWERUPS = [(pup, pup in VISIBLE) for pup in ALL]

# Only installed for users created on the command line
TAP_PUPS = [
    (Adminup, False)
    ]

STOREUPS = (StoreRealm, BoxOffice, ControlPanel, Backup, MeterStoreup)

def importmessages(avatar, directories, mailboxes, pop3g, imap4g):
    if pop3g:
        u, p, h, P = pop3g.split(':')
        def _():
            IGrabberAdder(avatar).addPopGrabber(u, p, h, int(P), '', 0, False, None)
        avatar.transact(_)
    if imap4g:
        u, p, h, P = imap4g.split(':')
        def _():
            IGrabberAdder(avatar).addIMAPGrabber(u, p, h, int(P), '', 0, True, None)
        avatar.transact(_)

    lup = ISearch(avatar)
    lup.indexer.setMergeFactor(500)
    md = IMIMEDelivery(avatar)
    for directory in directories:
        mp = md.createMIMEReceiver(True)
        for df in os.listdir(directory):
            df = opj(directory, df)
            if isfile(df):
                try:
                    m = mp.feedFileNow(file(df))
                except:
                    log.err()
                else:
                    avatar.transact(m.setMessageOrigin, "file://" + abspath(df))

    for m in mailboxes:
        mp = md.createMIMEReceiver(True)
        mbox = mailbox.PortableUnixMailbox(file(m))
        n = 0
        for msg in mbox:
            mobj = mp.feedStringNow(str(msg) + '\r\n' + msg.fp.read())
            avatar.transact(mobj.setMessageOrigin, "mbox://%s#%d" % (abspath(m), n))
            n += 1

    lup.indexer.optimize()
    lup.indexer.close()

def makeService(config):
    stor = Store(config['storeID'], config['datadir'], config['msgcal'])

    s = service.MultiService()

    storeService = StorageService(stor)
    storeService.setServiceParent(s)

    self = foo()

    def _():
        sScheduler = SchedulerService(stor)
        ps = IPowerStation(stor)
        for storeup in STOREUPS:
            ps.installPowerup(storeup)
        sRealm = FwdRealm(portal.IRealm(stor).referenceTo())
        sRealm.addRefusalRelation(provisionRelation)
        sChecker = FwdChecker(ICredentialsChecker(stor).referenceTo())
        # Non-transactional code follows.
        webPortal = self.wp = portal.Portal(sRealm)
        webPortal.registerChecker(sChecker)
        webPortal.registerChecker(AllowAnonymousAccess())
        sScheduler.setServiceParent(s)
    stor.transact(_)

    def makeDomains():
        cc = ICredentialsChecker(stor)
        try:
            topDomain = cc.getDomain()
        except KeyError:
            # No default domain, so make one.
            cc.addDomain(config['domain'])

    stor.transact(makeDomains)

    def _():
        IPowerStation(stor).installPowerup(Sysup)
    stor.transact(_)

    for username, password, directories, mailboxes in config.users:
        a = stor.transact(makeUser, stor, username, password, POWERUPS + TAP_PUPS, False)
        importmessages(a, directories, mailboxes, config['pop3g'], config['imap4g'])

    # Note: we need different portals because different protocols use different
    # checkers (as per Exarkun's request for different credentials for
    # different levels of security, even if they're all passwords)

    site = root.QuotientSite(
        guard.SessionWrapper(
            guard.UsernamePasswordWrapper(
                self.wp,
                callback=utility.loginCallback,
                errback=utility.badLoginCallback
            )
        )
    )

    sipport = int(config['sipport'])
    sipd = sip.RegisterProxy(config['domain'], sipport)
    if config['asteriskport']:
        sipregistry = sip.ForwardingRegistry([config['domain']], stor, config['asteriskport'])
    else:
        sipregistry = sip.MultiDomainRegistry([config['domain']])
    sipd.portal = self.wp
    sipservice = sip.SIPServer(sipport, sipd, sipregistry, sipregistry)
    sipservice.setName("sip registrar")
    sipservice.setServiceParent(s)

    auths = {"CRAM-MD5": CramMD5Credentials}

    pop3d = POP3Factory(self.wp, None)
    smtpd = ESMTPInFactory(stor, self.wp, auths, None)
    imapd = IMAP4Factory(self.wp, auths, None)

    INTERNET = [('webport', site), ('pop3port', pop3d),
                ('smtpport', smtpd), ('imap4port', imapd)]

    for (inetd, inetf) in INTERNET:
        strports.service(config[inetd], inetf).setServiceParent(s)

    regadapt.AdapterRegistrationService().setServiceParent(s)

    return s
