# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

"""
Test our code coverage.
"""

import time, sys, os
from os.path import join as opj

from twisted.python import reflect
from twisted.python.util import sibpath
from twisted.trial import unittest, reporter

import quotient
from quotient.benchmarks import coverage

tests = ['quotient.test', 'nevow.test', 'atop.test']
modules = ['quotient', 'nevow', 'atop']

class CoverageTest(unittest.TestCase):
    def benchmarkCoverage(self):
        coverage.start()
        subprocessCoverage = os.path.abspath(self.mktemp())
        t = unittest.TestSuite()
        for test in tests:
            t.addPackage(test)
        for testAdapter in t.tests:
            testAdapter.testClass.COVERAGE = opj(sibpath(quotient.__file__, 'benchmarks'), 'coverage.py')
            testAdapter.testClass.COVERAGE_FILE = subprocessCoverage

        t.run(reporter.MinimalReporter(sys.stdout))
        coverage.stop()
        for i in range(5):
            if os.path.exists(subprocessCoverage):
                break
            time.sleep(1)
        else:
            pass
            # raise unittest.FailTest, "Subprocess did not publish coverage statistics"
        # coverage.merge(subprocessCoverage)

        exclude = ['test', 'benchmarks', '_trial_temp']
        totLines = 0
        uncoveredLines = 0
        for module in modules:
            for path, subdirs, filenames in os.walk(os.path.dirname(os.path.abspath(reflect.namedModule(module).__file__))):
                subdirs[:] = [d for d in subdirs if d not in exclude]
                filenames = [f for f in filenames if f.endswith('.py')]
                for f in filenames:
                    module = os.path.join(path, f)
                    fname, statements, uncovered, _ = coverage.analysis(module)
                    print fname, 'contributing', len(statements), len(uncovered), 1 - float(len(uncovered)) / (len(statements) + 1)
                    totLines += len(statements)
                    uncoveredLines += len(uncovered)
        pctCovered = (1.0 - (uncoveredLines / float(totLines))) * 100
        self.recordStat({'testCoverage': (pctCovered, "Coverage")})
    benchmarkCoverage.type = 'percent'
    benchmarkCoverage.name = 'Coverage'
