# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

import time

from atop import sched
from atop import store
from atop.test import unittest

from quotient.test.emaildata import test_messages
from quotient.test import helper

from quotient.mimemessage import messageFromStructure
from quotient.gcup import GarbageCollector, ICollector

class GarbageCollection(unittest.TestCase, helper.ProvisionerMixin):
    storePowerups = [sched.SystemScheduler]

    def dbSetUp(self):
        self.avatar = self.createAvatar([GarbageCollector])

    def dbTearDown(self):
        dc = sched.IScheduler(self.store).nextCall
        dc and dc.cancel()

    def dbtestTimeThreshold(self):
        c = ICollector(self.avatar)

        spam = c.spamRef.getItem()
        spamTrash = c.trashSpamRef.getItem()
        trash = c.trashRef.getItem()

        now = sched.IScheduler(self.avatar).clock()
        day = 60 * 60 * 24

        for i in range(-14, 0):
            when = now + (i * day) + 60
            m = messageFromStructure(self.avatar, [], "")
            m.setReceived(time.gmtime(when))
            spam.addItem(m)
            
        self.assertEquals(len(list(spam.queryIndex())), 14)
        self.assertEquals(len(list(spamTrash.queryIndex())), 0)
        self.assertEquals(len(list(trash.queryIndex())), 1)

        c.run()

        self.assertEquals(len(list(spam.queryIndex())), 7)
        self.assertEquals(len(list(spamTrash.queryIndex())), 7)
        self.assertEquals(len(list(trash.queryIndex())), 1)

        c.empty()

        self.assertEquals(len(list(spam.queryIndex())), 7)
        self.assertEquals(len(list(spamTrash.queryIndex())), 0)
        self.assertEquals(len(list(trash.queryIndex())), 1)

        m = spam.queryIndex().next()
        spam.removeItem(m)
        trash.addItem(m)

        c.empty(all=True)

        self.assertEquals(len(list(spam.queryIndex())), 6)
        self.assertEquals(len(list(spamTrash.queryIndex())), 0)
        self.assertEquals(len(list(trash.queryIndex())), 1)
