# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

"""
Benchmarks for the user-visible delay of different actions.
"""

import time
from os.path import join as opj

from atop import store
from atop import sched
from atop import credup
from atop import powerup

from twisted.trial import unittest
from twisted.protocols import pop3
from twisted.protocols import imap4

from quotient.proto import popup
from quotient.proto import imapup
from quotient import mimemessage
from quotient import provisioner
from quotient import searchup

from quotient.test import emaildata

class BigMailbox:
    _av = None
    def _makeAvatar(self):
        p = self.mktemp()
        s = store.Store(0, opj(p, "db"), opj(p, "mail"))
        store.StorageService(s)
        def _():
            a = credup.Avatar(s)
            ps = powerup.IPowerStation(a)
            pv = ps.installPowerup(provisioner.Provisioner)
            provisioner.oneYearService.grantTo(pv)
            map(ps.installPowerup, self.avatarPowerups)

            return a
        a = s.transact(_)
        mrf = mimemessage.IMIMEDelivery(a).createMIMEReceiver
        lc = self.MESSAGES / len(emaildata.test_messages)
        def _():
            for m in emaildata.test_messages:
                mrf(True).feedStringNow(m)
                self.actualMessages = lc * len(emaildata.test_messages)
        for i in range(lc):
            a.transact(_)
        return a

    def avatar(self):
        if self._av is None:
            self._av = self._makeAvatar()
        return self._av

class Latency:

    LABEL = "Seconds_to_open_%s_with_%d_messages"

    avatarPowerups = [popup.Popup, imapup.IMAPAccountUp]

    def tearDown(self):
        nc = sched.IScheduler(self.avatar().store).nextCall
        nc and nc.cancel()
        nc = searchup.ISearch(self.avatar()).checkpointDefer
        nc and nc.cancel()

    def benchmarkPOP3OpenLatency(self):
        a = self.avatar()

        def _():
            b = time.time()
            mbox = pop3.IMailbox(a)
            mbox.login()
            e = time.time()
            self.assertEquals(len(mbox.msgs), self.actualMessages)
            return b, e
        b, e = a.transact(_)

        stat = e - b
        self.recordStat({"pop3-delay": (stat, self.LABEL % ("POP3", self.actualMessages))})

    def benchmarkIMAP4OpenLatency(self):
        a = self.avatar()

        b = time.time()
        poolbox = imap4.IAccount(a).select("inbox")
        e = time.time()

        self.assertEquals(len(poolbox.sequence), self.actualMessages)

        stat = e - b
        self.recordStat({"imap4-delay": (stat, self.LABEL % ("IMAP4", self.actualMessages))})

class Latency100(Latency, unittest.TestCase, BigMailbox):
    MESSAGES = 100

class Latency1000(Latency, unittest.TestCase, BigMailbox):
    MESSAGES = 1000

class Latency3000(Latency, unittest.TestCase, BigMailbox):
    MESSAGES = 3000
