# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

from os.path import join as opj

from atop.store import Store, SubStore
from atop.powerup import IPowerStation
from atop.test import unittest

from quotient.test import emaildata
from quotient.test import helper
from quotient.mimemessage import messageFromStructure
from quotient.smarts.list import MailingListFiler
from quotient import smartenup
from quotient import cleanup
from quotient import searchup

mailingListMessage = ([("X-Mailman-Version", "2.0.11"),
                       ("List-Help", "<mailto:twisted-python-request@twistedmatrix.com?subject=help>"),
                       ("List-Post", "<mailto:twisted-python@twistedmatrix.com>"),
                       ("List-Subscribe", "<http://twistedmatrix.com/cgi-bin/mailman/listinfo/twisted-python>"),
                       ("List-Id", "Twisted Python discussion stuff <twisted-python.twistedmatrix.com>"),
                       ("List-Unsubscribe", "<http://twistedmatrix.com/cgi-bin/mailman/listinfo/twisted-python>"),
                       ("List-Archive", "<http://twistedmatrix.com/pipermail/twisted-python/>")],
                       "Here is body content.")

class TestListup(unittest.TestCase, helper.ProvisionerMixin):
    def dbSetUp(self):
        self.substore = self.createAvatar([smartenup.Smartenup])
        self.listUp = IPowerStation(self.substore).installPowerup(MailingListFiler)
        msg = messageFromStructure(self.substore, *mailingListMessage)
        cleanup.IClean(self.substore).cleanRef.getItem().addItem(msg)

    def dbTearDown(self):
        nc = searchup.ISearch(self.substore).checkpointDefer
        nc and nc.cancel()

    def dbtestMailListAutoPoolCreation(self):
        lp = self.listUp.extractPool()
        thisPool = lp.getIndexItem('name', 'twisted-python.twistedmatrix.com', None)
        self.failUnless(thisPool)

    def dbtestMailListAutoFiling(self):
        lp = self.listUp.extractPool()
        thisPool = lp.getIndexItem('name', 'twisted-python.twistedmatrix.com', None)
        poolLen = len(list(thisPool.queryIndex()))
        self.assertEquals(poolLen, 1)
