# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

from os.path import join as opj
import os
import cStringIO

from atop.store import Store, SubStore, Item, Pool, StorageService
from atop.powerup import IPowerStation
from atop.sched import IScheduler, SchedulerService, SubScheduler
from atop.test import unittest

from quotient import mimemessage
from quotient.mimemessage import MIMEMessageReceiver, IMIMEDelivery
from quotient import grabbers
from quotient.test import emaildata, helper

from itertools import izip

# from email.Message import Message as PyEmailMessage
from email import message_from_string

from twisted.cred.portal import IRealm, Portal
from twisted.cred.checkers import ICredentialsChecker
from twisted.protocols.pop3 import IMailbox, POP3
from twisted.internet import protocol
from twisted.internet import defer

from quotient.test.emaildata import test_messages

class POP3TestMailbox:
    __implements__ = IMailbox
    def listMessages(self, index=None):
        if index is None:
            return map(len, test_messages)
        else:
            return len(test_messages[index])
    def getMessage(self, index):
        return cStringIO.StringIO(test_messages[index])
    def getUidl(self, index):
        return str(index)
    def deleteMessage(self, index):
        return
    def undeleteMessages(self):
        return
    def sync(self):
        return

class POP3TestRealm:
    __implements__ = IRealm
    def requestAvatar(self, avatarId, mind, *interfaces):
        return interfaces[0], POP3TestMailbox(), lambda : None

from twisted.internet import protocol

class POP3TestFactory(protocol.Factory):
    def __init__(self, portal):
        self.portal = portal

    def buildProtocol(self, addr):
        p3 = POP3()
        p3.portal = self.portal
        return p3

from twisted.cred import credentials

class POP3TestChecker:

    __implements__ = ICredentialsChecker
    credentialInterfaces = (credentials.IUsernamePassword,
                            credentials.IUsernameHashedPassword)

    def requestAvatarId(self, credentials):
        return 'anybody'

class MIMEMessageTestCase(unittest.TestCase, helper.ProvisionerMixin):

    def setUp(self):
        dbpath = self.mktemp()
        self.store = Store(0, opj(dbpath, "db"), opj(dbpath, "files"))
        StorageService(self.store)
        def _():
            self.svc = SchedulerService(self.store)
            self.substore = self.createAvatar([SubScheduler])
            self.svc.startService()
        self.store.transact(_)

    def tearDown(self):
        self.svc.stopService()

    def testBasicParsing(self):
        L = []
        for m in emaildata.test_messages:
            MIMEMessageReceiver(self.substore, L.append).feedStringNow(m)
        self.assertEquals(len(L),
                          len(emaildata.test_messages))
        L2 = [message_from_string(M) for M in emaildata.test_messages]

        for OurM, TheirM in zip(L, L2):
            if OurM.is_multipart():
                self.assertEquals(OurM.getPartByID(1), OurM.children[0])
                self.assertEquals(OurM.getPartByID(0), OurM)
                self.assertEquals(1, OurM.children[0].mimeID)
                self.assertEquals(0, OurM.mimeID)
            for ourPart, theirPart in izip(OurM.walk(), TheirM.walk()):
                self.assertEquals(
                    ourPart['subject'],
                    theirPart['subject'])
                self.assertEquals(
                    ourPart.is_multipart(),
                    theirPart.is_multipart())
                if not ourPart.is_multipart():
                    ourBody = ourPart.get_payload(decode=True)
                    theirBody = theirPart.get_payload(decode=True)
                    if ourBody != theirBody:
                        if ourBody[-1] == '\n':
                            if ourBody[:-1] == theirBody:
                                continue
                        raise '?'
                else:
                    self.assertEquals(len(ourPart.children),
                                      len(theirPart._payload))

    def testReceivingMessages(self):
        from quotient import grabbers
        from atop import powerup
        from twisted.internet import reactor
        prtl = Portal(POP3TestRealm())
        prtl.registerChecker(POP3TestChecker())
        p3f = POP3TestFactory(prtl)
        prt = reactor.listenTCP(0, p3f, interface="127.0.0.1")
        portnum = prt.getHost()[2]
        # reactor.connectTCP("127.0.0.1", portnum, )
        def _():
            pup = powerup.IPowerStation(self.substore).installPowerup(grabbers.GrabberPowerup)
            grab = pup.addPopGrabber("anybody", "anybody", "127.0.0.1", portnum, "anybody@localhost", 100, False, None)
            return grab
        g = self.substore.transact(_)
        d = self.substore.transact(g.run)
        d.addCallback(lambda x: reactor.crash())
        reactor.run()
        def _2():
            # whiiiiiite boooooooox
            # powerup.IPowerStation(self.substore).installPowerup(mimemessage.EmailPowerup)
            arrivalPool = IMIMEDelivery(self.substore).arrivalRef.getItem()
            self.assertEquals(len(list(arrivalPool.queryIndex("uid"))),
                              len(test_messages))
        self.substore.transact(_2)

class GrabberSchedulingTestCase(unittest.TestCase, helper.ProvisionerMixin):

    def dbSetUp(self):
        self.svc = SchedulerService(self.store)
        self.substore = self.createAvatar([grabbers.GrabberPowerup])
        self.gp = grabbers.IGrabberAdder(self.substore)
        self.svc.startService()

    def dbTearDown(self):
        self.svc.stopService()

    def dbtestPausing(self):
        f = self.gp.addPopGrabber("user", "pass", "host", 12345, "user@host", 0, 0, 0)
        f.paused = True

        when = IScheduler(self.store).clock()
        d = self.store.transact(f.run)
        self.assertEquals(f.paused, True)
        self.assertEquals(d, None)
        self.assertEquals(f.currentProtocol, None)

    def dbtestMultipleRuns(self):
        from twisted.internet import reactor
        sf = ServerFactory()
        sf.protocol = protocol.Protocol
        p = reactor.listenTCP(0, sf).getHost()[2]

        cf = self.gp.addPopGrabber("user", "pass", "localhost", p, "user@localhost", 0, 0, 0)
        d1 = cf.run()
        d2 = cf.run()
        self.assertIdentical(d1, d2)

        unittest.deferredResult(sf.connected)
        self.failUnless(sf.connMade)

class FlaggedItem(Item):
    def __init__(self, store, flags):
        Item.__init__(self, store)
        self.flags = flags
    def index_flags(self, pool):
        return self.flags

class Flags(unittest.TestCase):
    def dbSetUp(self):
        self.avatar = self.createAvatar(())
        self.pool = Pool(self.avatar)
        self.pool.addIndex(mimemessage.FlagsIndex())

    def dbtestFlagCounter(self):
        a = self.pool.addItem
        a(FlaggedItem(self.avatar, 0))
        a(FlaggedItem(self.avatar, 1))
        a(FlaggedItem(self.avatar, 1 | 2))
        a(FlaggedItem(self.avatar, 2 | 4))
        a(FlaggedItem(self.avatar, 1 | 4))
        a(FlaggedItem(self.avatar, 1 | 8))
        a(FlaggedItem(self.avatar, 4 | 16))

        f = mimemessage.numberFlagged

        self.assertEquals(f(self.pool, 1), 4)
        self.assertEquals(f(self.pool, 2), 2)
        self.assertEquals(f(self.pool, 4), 3)
        self.assertEquals(f(self.pool, 8), 1)
        self.assertEquals(f(self.pool, 16), 1)
        self.assertEquals(f(self.pool, 0), 0)
        self.assertEquals(f(self.pool, 31), 6)

class ServerFactory(protocol.ServerFactory):
    connMade = False
    def __init__(self):
        self.connected = defer.Deferred()

    def buildProtocol(self, addr):
        self.connMade = True
        self.connected.callback(self)
        return protocol.ServerFactory.buildProtocol(self, addr)
