
from os.path import join as opj

from atop.store import Store
from atop.credup import Avatar
from atop.powerup import IPowerStation
from atop.test import unittest
from atop.sched import SystemScheduler, IScheduler

from quotient.smarts import person
from quotient.mimemessage import EmailPowerup, IMIMEDelivery
from quotient.test import helper
from quotient import searchup

fromalice = [
'''From: alice@fooco.com
To: you
Subject: stuf!

This is a message from alice.
''',
'''From: "Alice Whatever" <alice@fooco.com>
To: "You!" <notalice@you>
Subject: YEs

This also a message from alice.
''']

notfromalice = [
'''From: alice@notfooco.com
To: you
Subject: NOT FROM ALICE.

This is not a message from alice.
''']


class PeopleTest(unittest.TestCase, helper.ProvisionerMixin):
    storePowerups = [SystemScheduler]

    def dbSetUp(self):
        a = self.jethro = self.createAvatar((EmailPowerup, person.PersonCollector))
        ppup = person.IPersonCollector(a)
        alice = self.alice = ppup.newPerson("Alice McExampleton")
        info = alice.newCard("VP of Marketing", "FooCo")
        info.setEmailAddress("alice@fooco.com")
        info.setPhoneNumber("123-456-7890")
        info = alice.newCard("McExampleton Family", "Mother of 6")
        info.setEmailAddress("alice@mcexampleton.funkyisp.ny.us")

    def dbTearDown(self):
        # nc = IScheduler(self.store).nextCall
        # nc and nc.cancel()
        nc = searchup.ISearch(self.jethro).checkpointDefer
        nc and nc.cancel()

    def dbtestFindingContact(self):
        ppup = person.IPersonCollector(self.jethro)
        pepool = ppup.getPeoplePool()
        alice = pepool.queryIndex().next()
        md = IMIMEDelivery(self.jethro)
        for msgt in fromalice + notfromalice:
            mr = md.createMIMEReceiver(True)
            mr.feedStringNow(msgt)
        self.assertEquals(alice, self.alice)
        self.failIf(len(fromalice) > 100,
                    "You're getting carried away with example messages.")
        msgs = list(alice.recentMessages(100))
        self.assertEquals(len(fromalice), len(msgs))
    # dbtestFindingContact.skip = "Need to fix 'pendingTimedCalls still pending'"

    def dbtestBurningCard(self):
        ppup = person.IPersonCollector(self.jethro)
        pepool = ppup.getPeoplePool()
        alice = pepool.queryIndex().next()
        c = alice.newCard("Helsing & Associates", "Vampire Hunter")
        amh = "alicem@helsing.divmod.com"
        c.setEmailAddress(amh, 'email')
        amh = "alicem@helsing.divmod.com"
        c.setEmailAddress(amh, 'work')
        self.assertEquals(list(pepool.queryIndex("email", startKey=amh, endKey=amh)), [alice])
        c.burn()
    # dbtestBurningCard.skip = "Need to fix 'pendingTimedCalls still pending'"
