# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

from atop import fwd
from atop import sched
from atop import powerup
from atop.test import unittest

from twisted.python import components
from twisted.internet import defer

from quotient import reset
from quotient import prefup
from quotient.proto import smtpout
from quotient.test import helper

class CapturingMailer(powerup.Powerup):
    __implements__ = (smtpout.IOutgoingMail,)

    sent = None

    def sendString(self, txt, attachments, to, from_, cc='', bcc='', subject=''):
        self.sent = txt, attachments, to, from_, cc, bcc, subject
        return defer.succeed("Some junk")

class Reset(unittest.TestCase, helper.ProvisionerMixin):

    storePowerups = [sched.SystemScheduler]

    def dbSetUp(self):
        self.avatar = self.createAvatar([reset.PasswordResetter, prefup.Prefup], name="janitor")
        self.mailer = CapturingMailer(self.avatar)
        self.avatar.setComponent(smtpout.IOutgoingMail, self.mailer)

    def dbTearDown(self):
        dc = sched.IScheduler(self.store).nextCall
        dc and dc.cancel()

    def testComponents(self):
        c = reset.IPasswordResetter(self.avatar)
        self.failUnless(components.implements(c, reset.IPasswordResetter))

    def dbtestRequestReset(self):
        r = reset.IPasswordResetter(self.avatar)
        a = self.createAvatar([prefup.Prefup], name="av5")

        prefup.IPrefs(a).setPref("emergencyContact", "foo@bar")

        result = unittest.deferredResult(r.requestReset(a))

        txt, attachments, to, from_, cc, bcc, subject = self.mailer.sent

        self.assertEquals(len(r.outstanding), 1)
        key, storeID = r.outstanding.items()[0]

        self.failIfEqual(txt.find(key), -1)
        self.assertEquals(storeID, a.storeID)

    def dbtestUnsetContact(self):
        r = reset.IPasswordResetter(self.avatar)
        a = self.createAvatar([prefup.Prefup], name="av5")
        self.assertRaises(reset.NoContactAddressDefined, r.requestReset, a)

    def dbtestPendingRequest(self):
        r = reset.IPasswordResetter(self.avatar)
        a = self.createAvatar([prefup.Prefup], "av5")

        prefup.IPrefs(a).setPref("emergencyContact", "foo@bar")

        unittest.deferredResult(r.requestReset(a))
        self.assertRaises(reset.ResetAlreadyPending, r.requestReset, a)

    def dbtestRecentRequest(self):
        r = reset.IPasswordResetter(self.avatar)
        a = self.createAvatar([prefup.Prefup], "av5")

        prefup.IPrefs(a).setPref("emergencyContact", "foo@bar")

        unittest.deferredResult(r.requestReset(a))

        # Simulate an actual reset
        r.outstanding.clear()
        del r.outstandingSorted[:]

        self.assertRaises(reset.ResetRecentlyRequested, r.requestReset, a)
