# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

from os.path import join as opj

from atop.store import Store
from atop.test import unittest

from quotient.test import emaildata, helper
from quotient.mimemessage import IMIMEDelivery
from quotient.cleanup import IClean, Cleanup
from atop.sched import SystemScheduler, IScheduler

class TestSpambayesup(unittest.TestCase, helper.ProvisionerMixin):

    storePowerups = [SystemScheduler]

    def dbSetUp(self):
        self.avatar = self.createAvatar([Cleanup])
        self.cleaner = IClean(self.avatar)
        self.guesser = self.cleaner.sbRef.getItem()
        self.md = IMIMEDelivery(self.avatar)

        for m in emaildata.test_messages:
            self.md.createMIMEReceiver(True).feedStringNow(m)

    def dbTearDown(self):
        # Clear out pending calls
        dc = IScheduler(self.store).nextCall
        dc and dc.cancel()

    def dbtestGuess(self):
        msgsInClean = list(self.guesser.cleanRef.getItem().queryIndex())
        self.failUnless(len(msgsInClean) >= 6, 'Expected at least 6 clean, got %s' % len(msgsInClean))

    def dbtestTrain(self):
        msgsInClean = self.guesser.cleanRef.getItem().queryIndex()
        for msg in msgsInClean:
            self.guesser.train(msg,'spam')
            self.guesser.train(msg,'notspam')

        # If we get through that it means *something* at least coverage
        # and spambayes tokenizer
