# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

from os.path import join as opj

from atop.credup import preauth

from twisted.trial import unittest

from quotient.tap2 import makeService
from quotient.searchup import ISearch

class Config(dict):
    users = []

class TapTest(unittest.TestCase):

    storeOpts = {'master-store': True, 'storeID': 0}
    appOpts = {'domain': 'localhost'}
    portOpts = {'pop3port': '0', 'imap4port': '0', 'smtpport': '0', 'webport': '0', 'sipport': '0', 'asteriskport': None}
    grabOpts = {'pop3g': None, 'imap4g': None}
    dirOpts = {'datadir': 'db', 'msgcal': 'files'}

    allOpts = {}
    allOpts.update(storeOpts)
    allOpts.update(appOpts)
    allOpts.update(portOpts)
    allOpts.update(grabOpts)
    allOpts.update(dirOpts)

    def setUp(self):
        dir = self.mktemp()
        opts = self.allOpts.copy()
        opts['datadir'] = opj(dir, opts['datadir'])
        opts['msgcal'] = opj(dir, opts['msgcal'])
        self.opts = opts

    def testMktap(self):
        config = Config(**self.opts)
        config.users = [('foo', 'bar', [], [])]
        s = makeService(config)
        
        store = s.getServiceNamed("atop.Store").store
        from atop import sched
        dc = sched.IScheduler(store).nextCall
        dc and dc.cancel()

        ISearch(preauth(store, "foo")).checkpointDefer.cancel()
        

    def testMasterStore(self):
        config = Config(**self.opts)
        config['storeID'] = 19
        s = makeService(config)
        ss = s.getServiceNamed("atop.Store")
        self.assertEquals(ss.store.storeID, 19)
        
        from atop import sched
        dc = sched.IScheduler(ss.store).nextCall
        dc and dc.cancel()
