# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

import datetime

from cPickle import loads, dumps
from twisted.trial.unittest import TestCase
from quotient.timezones import zones, UTC, Local, Eastern
from quotient import timezones

class ZoneTestCase(TestCase):

    def testPickling(self):
        # I just needed to make sure this worked, and I figure this is a better
        # test case than just typing junk interactively - don't take it too
        # seriously -glyph
        assert Local in zones
        assert UTC in zones
        assert Eastern in zones
        tup = tuple(zones)
        s = dumps(tup)
        tup2 = loads(s)
        for i, x in enumerate(tup):
            self.assertIdentical( tup2[i], x )

    def testDaysInMonth(self):
        fmt = "days in %s != %d"
        data = [(1, 31),  (2, 28),  (3, 31),  (4, 30),
                (5, 31),  (6, 30),  (7, 31),  (8, 31),
                (9, 30),  (10, 31), (11, 30), (12, 31)]

        for (m, ndays) in data:
            tz = datetime.datetime(2001, m, 1)
            self.assertEquals(ndays, timezones.days_in_month(tz), fmt % (tz, ndays))
            tz = datetime.datetime(1900, m, 1)
            self.assertEquals(ndays, timezones.days_in_month(tz), fmt % (tz, ndays))
            
        data = [(1, 31),  (2, 29),  (3, 31),  (4, 30),
                (5, 31),  (6, 30),  (7, 31),  (8, 31),
                (9, 30),  (10, 31), (11, 30), (12, 31)]

        for (m, ndays) in data:
            tz = datetime.datetime(1996, m, 1)
            self.assertEquals(ndays, timezones.days_in_month(tz), fmt % (tz, ndays))
            tz = datetime.datetime(2000, m, 1)
            self.assertEquals(ndays, timezones.days_in_month(tz), fmt % (tz, ndays))

    def testIsLeapYear(self):
        for i in range(1904, 2100):
            tz = datetime.datetime(i, 1, 1)
            if i % 4 == 0:
                self.failUnless(timezones.is_leap_year(tz), "%s is not a leap year??" % (tz,))
            else:
                self.failIf(timezones.is_leap_year(tz), "%s is a leap year??" % (tz,))
