# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.


from quotient.proto import smtpout
from quotient import mimemessage
from quotient import taglib

from nevow import freeform
from nevow import formless

from quotient.mimemessage import MIMEMessage, headerParams
from quotient import prefup
from quotient.web import root
from quotient.taglib import *


class IComposer(formless.TypedInterface):
    def sendMail(self,
        request=formless.Request(),
        to=formless.String(label="To:"),
        cc=formless.String(label="CC:"),
        #bcc=formless.String(label="BCC:"),
        replyTo=formless.String(label="Reply to:"),
        subj=formless.String(label="Subject:"),
        text=formless.Text(label="Body:"),
        attachment=formless.FileUpload(label="Attachment:")):
        """Compose

        Send Mail to someone.
        """
        return None
    sendMail = formless.autocallable(sendMail, action="Send", revealed=True)


def reSubject(m):
    newsubject = m['subject'] or ''
    if not newsubject.lower().startswith('re:'):
        newsubject = 'Re: ' + newsubject
    return newsubject


def quoteBody(m):
    newtext = m.getDisplayPart().get_payload().split('\n')
    return [ '\n> '.join(newtext) ]


def aspectProperty(aspect, name):
    return property(
        lambda self: getattr(aspect(self.original), name),
        lambda self, val: setattr(aspect(self.original), name, val)
    )


class Compose(root.Base, freeform.Configurable, object):
    adaptsAvatar = True

    __implements__ = IComposer, freeform.Configurable.__implements__,

    def __init__(self, original):
        root.Base.__init__(self, original)
        freeform.Configurable.__init__(self, original)

    document = taglib.page("")[
    freeform.configure
]

    def setDefaults(self, req, msg=None, flags = []):
        if flags:
            def _():
                msg.touch()
                for flag in flags:
                    msg.changeFlag(flag)
            msg.store.transact(_)

        from nevow import freeform
        return freeform.IFormDefaults(req).getAllDefaults('sendMail')

    def do_reply(self, req, msg):
        formContents = self.setDefaults(req, msg, flags=[MIMEMessage.ANSWERED])
        formContents['to'] = msg['from'] or ''
        formContents['subj'] = reSubject(msg)
        formContents['replyTo'] = msg.addressedTo or prefup.IPrefs(self.original).getPref('replyto') or ''
        newtext = quoteBody(msg)

        origdate, origfrom = msg['date'], msg['from']
        if not origdate:
            origdate = "an indeterminate time in the past"
        if not origfrom:
            origfrom = "someone who chose not to be identified"
        replyhead = 'On %s, %s wrote:\n>' % (origdate.strip(), origfrom.strip())
        formContents["text"] = replyhead + ('\n> '.join(newtext))

    def do_forward(self, req, msg):
        formContents = self.setDefaults(req, msg, flags=[MIMEMessage.FORWARDED])
        newsubject = reSubject(msg)
        formContents["subj"] = newsubject
        formContents["replyTo"] = msg.addressedTo or prefup.IPrefs(self.original).getPref('replyto') or ''
        newtext = quoteBody(msg)

        reply = ["\nBegin forwarded message:\n"]
        reply.extend(['%s: %s' % (x, msg[x]) for x in ['From', 'Date', 'To', 'Subject', 'Reply-to'] if msg[x]])
        reply.append('')
        reply.extend(newtext)

        formContents["text"]='\n> '.join(reply)

    def do_redirect(self, req, msg):
        formContents = self.setDefaults(req, msg, flags=[MIMEMessage.REDIRECTED])
        newsubject = reSubject(msg)
        formContents["subj"] = newsubject
        formContents["replyTo"] = msg['from'] or ''

        fullText = [msg.getDisplayPart().get_payload().splitlines()]

        for i, x in enumerate(fullText):
            fullText[i] = '\n'.join(x)
        formContents["text"]='\n'.join(fullText)

    def render(self, request):
        for act in ['reply', 'forward', 'redirect']:
            msgnum = request.args.get(act, [''])[0]
            if msgnum:
                msg = self.original.getItemByID(int(msgnum))
                getattr(self, 'do_%s' % act, lambda req, msg: None)(request, msg)

        return root.Base.render(self, request)

    replyTo = aspectProperty(prefup.IPrefs, 'replyto')

    def sendMail(self, request,to,cc,replyTo,subj,text,attachment):
        # attachment is a FieldStorage instance, ewww
        bcc = ''
        if attachment.value:
            attachments = [
                (attachment.value,
                 attachment.headers.get('content-type') or 'application/octet-stream',
                 headerParams(attachment.headers['content-disposition'] or '')[1].get('filename') or 'Attachment'),
                ]
        else:
            attachments = []
        smtpout.IOutgoingMail(self.original).sendString(text, attachments,
                                                        to, replyTo, cc, bcc, subj)

    postLocation = 'compose'
