# -*- test-case-name: quotient.test.test_webui -*-

import os
import sys

from atop.test import unittest

import quotient
from quotient.web import newclient

from twisted.internet import reactor
from twisted.internet import defer
from twisted.internet import protocol

from twisted.python import log
from twisted.runner import procutils

import urllib

class ChildProcessProtocol(protocol.ProcessProtocol):
    def __init__(self, doneLoading):
        self.doneLoading = doneLoading

    def outReceived(self, data):
        log.msg('child process outRecieved: %s' % data)
        #print "OUT: ", data
        if 'set uid/gid' in data:
            d, self.doneLoading = self.doneLoading, None
            if d:
                d.callback("twistd ready")

    def errReceived(self, data):
        #print "ERR: ", data
        # something using twisted.internet.app is screwing this up
        if 'DeprecationWarning' in data:
            return
        log.msg('child process errRecieved: %s' % data)
        d, self.doneLoading = self.doneLoading, None
        if d:
            d.errback(Exception(data))

    def processEnded(self, status):
        log.msg('childProcess processEnded with status: %s' % status)
        d, self.doneLoading = self.doneLoading, None
        if d:
            d.callback("It's done")


env = {}
if os.environ.get('PATH'):
    env['PATH'] = os.environ['PATH']
env['PYTHONPATH'] = os.path.split(quotient.__path__[0])[0]
if os.environ.get('PYTHONPATH'):
    env['PYTHONPATH'] += ':' + os.environ['PYTHONPATH']

log.msg("env: %s" % env)

TWISTD = (procutils.which("twistd2.3") + procutils.which("twistd"))[0]
MKTAP = (procutils.which("mktap2.3") + procutils.which("mktap"))[0]
PYTHON = (procutils.which("python2.3") + procutils.which("python"))[0]

class _WaitToStop(protocol.Factory):
    def __init__(self):
        self.deferred = defer.Deferred()
    def stopFactory(self):
        self.deferred.callback('foo')


class FunctionalBase(unittest.TestCase):
    caseMethodName = ''
    skip = False
    didSetUpClass = False

    COVERAGE = None
    COVERAGE_FILE = None

    def _initPorts(self):
        # heuristic, but bleh
        self.portArgs = []
        self.prts = prts = {}
        portObjs = []
        dL = []
        for prt in ['web','pop3','smtp','imap4','sip']:
            dfac = _WaitToStop()
            dL.append(dfac.deferred)
            t = reactor.listenTCP(0, dfac)
            inet, hostname, portno = t.getHost()
            self.portArgs.extend(['--%sport' % prt, str(portno)])
            portObjs.append(t)
            prts[prt] = portno
        for t in portObjs:
            t.stopListening()
        unittest.deferredResult(defer.DeferredList(dL))

    def setUpClass(self):
        if self.skip:
            return
        self.curId = 0
        self.dir = self.mktemp()
        tapfile = os.path.abspath(self.dir + '/quotient.tap')
        os.mkdir(self.dir)
        open(os.path.join(self.dir, '_urltrail.html'), 'a').write('<html><head><title>urltrail</title></head><body><table>')
        self.opener = newclient.opener()
        mktapDone = defer.Deferred()
        mktapProcess = ChildProcessProtocol(mktapDone)
        self._initPorts()
        #print "DEBUG:", self.portArgs, env, self.dir
        args = ['python', MKTAP, '--append', tapfile, 'quotient'] + self.portArgs
        reactor.spawnProcess(mktapProcess, PYTHON, args, env, self.dir)
        unittest.deferredResult(mktapDone)
        print "DEBUG: listdir ", os.listdir(self.dir)
        done = defer.Deferred()
        self.childProcess = ChildProcessProtocol(done)

        bin = PYTHON
        if self.COVERAGE:
            args = ['python', self.COVERAGE, "-x", TWISTD, "-nof", tapfile]
            env['COVERAGE_FILE'] = self.COVERAGE_FILE
        else:
            args = ['python', TWISTD, "-nof", tapfile]

        reactor.spawnProcess(self.childProcess, bin, args, env, self.dir)
        unittest.deferredResult(done)
        self.didSetUpClass = True
        self.doGet('/')

    def tearDownClass(self):
        if not self.didSetUpClass:
            return

        killDone = defer.Deferred()
        reactor.spawnProcess(ChildProcessProtocol(killDone), 'kill', ['kill', open(os.path.join(self.dir, 'twistd.pid')).read()], env, self.dir)
        open(os.path.join(self.dir, '_urltrail.html'), 'a').write('</table></body></html>')
        unittest.deferredResult(killDone)

    def doGet(self, path, args={}, timeout=1, skipRestOnError=True):
        url = ("http://localhost:%s" % self.prts['web']) + path
        def recordresult(result):
            #print result
            comment = "<!--result from url: %s-->\n\n" % url
            open(os.path.join(self.dir, 'urltrail'), 'a').write('%s: %s\n' % (self.curId, url))

            ########################################
            # WARNING: THIS MAY BE VERY VERY EVIL!!!
            callingFunc = None
            
            from inspect import currentframe, getouterframes
            try:
                oflist = getouterframes(currentframe())
                for n in oflist:
                    if 'test_' in n[3]:
                        callingFunc = n[3]
            finally:
                del oflist

            open(os.path.join(self.dir, '_urltrail.html'), 'a').write('<tr><td><a href="%s.html">%s</a></td>: <td>%s</td></tr>\n' % (self.curId, callingFunc, url))
            ########################################
            
            open(os.path.join(self.dir, '%s.html' % self.curId), 'w').write(''.join([comment,result]))
            self.curId += 1
            return result
        def dealWithError(error):
            #import pdb;pdb.set_trace() 
            #print dir(error)
            #print error.value
            #for k,v in error.value.__dict__.iteritems():
                #print "k:%s, v:%s" % (k,v)
            if error.value.status == '404':
                print "-" * 60
                print "404 ERROR IN functional.dealWithError \n\n"
                print "request information that caused 404 follows:\n\n"
                for k,v in error.value._request.__dict__.iteritems():
                    print "%s: %s" % (k, v)
                print "-" * 60
                print "\n\n"
            if not skipRestOnError:
                if hasattr(error, 'tb') and error.tb is not None:
                    import traceback
                    traceback.print_tb(error.tb)
                if isinstance(error.value, newclient.Response):
                    self.skip = "Functional test failed, skipping rest of sequence: %s" % error.value.message
                    log.err('functional.dealWithError skipping rest: %s' % error.value.message)
                    return error.value.message
                log.err('functional.dealWithError: %s' % error)
                return error
            else:
                log.msg('trapped exception')
                error.trap(Exception)
                log.msg(error.value)
                return None
        if args:
            argumentString = '?'
            for argKey, argValues in args.items():
                if isinstance(argValues, str):
                    argValues = [argValues]
                for val in argValues:
                    argumentString += '%s=%s&' % (urllib.quote(argKey), urllib.quote(val, safe=''))
                    log.msg('doGet argumentString: %s' % argumentString)
            argumentString = argumentString[:-1]
        else:
            argumentString = ''
        log.msg("doGet newclient.Request(%s%s)" % (url, argumentString))
        d = self.opener.open(
            newclient.Request(url + argumentString)
        ).addCallback(newclient.read
        ).addCallback(recordresult
        ).addErrback(dealWithError)

        self.result = ''
        self.result = unittest.deferredResult(d, timeout)

    def assertInResult(self, what, msg=None):
        self.assertIn(what, self.result, msg)

    def assertNotInResult(self, what, msg=None):
        self.failIfIn(what, self.result, msg)
