# -*- test-case-name: quotient.test.test_tdb -*-
# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.
##

import math
from itertools import islice, chain, repeat
from pprint import pformat

from urllib import urlencode

from atop import store

from twisted.python import reflect
from twisted.python import log
from twisted.python import components

from twisted.web import resource

from quotient.web import root
from quotient import taglib
from quotient.taglib import getAttribute, eitherOr, name, anId, nameOrId
from quotient import iquotient
from quotient.prefup import IPrefs

from nevow.iwoven import ISerializable, IRendererFactory, IRequest
from nevow.renderer import flatten, RenderFactory
from nevow import freeform
from nevow import formless

from nevow.tags import *

from atop.store import INDEX, STORE_ID

class TabularPool(root.TabularBase):
    def render_unseen(self, context, data):
        from quotient.mimemessage import MIMEMessage
        unseen = 0
        c = 0
        i = data.queryIndex('flags', result=INDEX, limit=1000)
        try:
            for flags in i:
                unseen += not (flags & MIMEMessage.SEEN)
                c += 1
        except KeyError:
            return ''
        return str(unseen) + (c == 1000 and '+' or '')
    root.column(render_unseen, True, 60)

class TabularNothing(root.TabularBase):
    storeID = 0
    name = property(lambda self: "A %s" % self.original.__class__.__name__)


def kindIcon(context, data):
    q = reflect.qual(data.__class__)
    return img(src=q, title=q, alt=q), q, " "


def anIntrospector(context, data):
    return pformat(data.__dict__)


CRAP = "float: left; width: %spx; height: 1.25em; overflow: hidden"
CHECKBOX = "float: left; height: 1.25em; width: 25px; overflow: hidden"


class ILastPool(components.Interface):
    """A marker interface which specifies a 2-tuple of (last-pool,
    last-index-name).
    """


def fileSpam(context, data):
    from quotient.web import webmail
    ava = context.locate(resource.IResource).avatar
    from quotient.cleanup import ISpambayes
    try:
        spam = ISpambayes(ava)
    except components.CannotAdapt:
        return span(_class="disabled")['Spam filtering not installed.']

    spamPool = spam.spamRef.getItem()
    cleanPool = spam.cleanRef.getItem()

    if data is spamPool:
        pl = 'Clean'
        style = 'file-clean'
    # double BARF
    elif data is cleanPool or data.name == 'Inbox':
        pl = 'Spam'
        style= 'file-spam'
    else:
        return ''

    try:
        guesses = context.locate(webmail.ISpamGuesses)
    except KeyError:
        top = context
        while top.parent is not None:
            top = top.parent
        guesses = {}
        top.remember(guesses, webmail.ISpamGuesses)
    items = '&'.join(
        ['items=%s' % x for x in guesses.get(pl,[])]
    )
    if items:
        return a(_class=style, href='%s?file=%s&%s' % (spam.storeID, pl, items))[ "File %s" % pl ]
    return span(style="color: #cdcdcd")[ "File %s" % pl ]

def href(*args, **kw):
    """Return an <a href='' /> link based on the current link with additional
    path elements (*args) and additional query arguments (**kw).
    """
    def _(context, data):
        argz = context.locate(IRequest).args.copy()
        argz.update(kw)
        path = '/'.join(map(str,args))
        return '%s?%s' % (path, urlencode(argz, True))
    return a(href=_)


from nevow import freeform
from nevow import formless



def Method(name, arguments=[]):
    return formless.MethodBinding(
        name,  ## The name of an action
        formless.Method(
            None,
            arguments,
            label = formless.labelAndDescriptionFromNameAndDocstring(name, '')
        )
    )



def Action(name):
    return Method(
        name,
        [formless.Argument('items', formless.List(formless.Object()))]
    )


class PoolBrowser(root.Base, freeform.Configurable):
    __implements__ = (root.Base.__implements__,
        freeform.Configurable.__implements__,
        IRendererFactory)

    def __init__(self, original, data=None, renderAddPool=False, suggestedView=None):
        root.Base.__init__(self, original)
        freeform.Configurable.__init__(self, original)
        self.data = data
        self.renderAddPool = renderAddPool
        self.suggestedView = suggestedView

    ## When used as a Resource, this document attribute will be used
    document = taglib.page(title=(name))[
    freeform.configure,
]

    ## When an instance of PoolBrowser is placed in a page,
    ## __call__ will be used
    def __call__(self, context, data):
        context.remember(self, IRendererFactory)
        return [
            invisible(data=self)[freeform.configure],
        ]

    def getView(self, context):
        ## hardcoded view -- for now
        if hasattr(self.original, 'suggestedView'):
            return self.original.suggestedView
        return "Table"

## Arbitrary user switching -- turn this on later
        request = context.locate(IRequest)
        self.view = request.args.get('view', [''])[0]
        if self.view not in self.availableViews:
            self.view = 'Table'
        return self.view

    def getBindingNames(self, context):
        ## Here, we switch the name we return based on the way the user wants to view this pool
        if self.suggestedView is not None:
            rv = [self.suggestedView]
        elif isinstance(self.data, list):
            rv = ['Table']
        else:
            rv = [self.getView(context)]
            if self.renderAddPool:
                rv += ['newPool']
        return rv

    def getBinding(self, context, name):
        ## figure out what the switch is set to, as above, and bind it
        if getattr(self, 'bind_%s' % name, None) is None:
            self.bind_Table(context)
        return freeform.Configurable.getBinding(self, context, name)

    def bind_Gallery(self, context):
        self.galleryContents = [invisible(data=self.original, renderer=self.render_gallery)]
        return formless.Property(
            "galleryContents",
            formless.List()
        )

    def bind_newPool(self, context):
        return Method(
            "newPool",
            [formless.Argument('poolName', formless.String())]
        )

    def newPool(self, poolName):
        new = store.Pool(self.original.store, name=poolName)
        self.original.addItem(new)

    def render_galleryItem(self, context, data):
        assert data

        rend = iquotient.ITabularRenderer(data)

        context.remember(rend, IRendererFactory)
        if hasattr(rend, 'render_thumbnail'):
            i = directive('thumbnail')
        else:
            i = a(href=getattr(data, 'storeID', ''))[ img(src="/images/icons/powerup.png") ]

        return table(border="0", height="100%")[
            tr[td[i]],
            tr[td(width="150")[
                span(style="float: left; width: 150px; height: 1.5em; overflow: hidden")[
                    directive('name')
                ]
            ]]
        ]

    def render_gallery(self, context, data):
        """forgive me lord, I know not what I do
        """
        indexIterator, nextPrev = self.locateIndexIterator(context, data)

        for numColumns in [3, 4, 5]:
            if not self.itemsPerPage % numColumns:
                break

        nextPrev = nextPrev(150 * numColumns)

        sliced = iter([x.original for x in self.indx])

        return nextPrev, table(border="0")[[
            tr[[
                td(data=item)[item and self.render_galleryItem or '']
                for item in islice(chain(sliced, repeat('')), 0, numColumns)
            ]]
            for i in range(int(math.ceil(self.itemsPerPage / numColumns)))
        ]], nextPrev

    def bind_Table(self, context):
        ## isinstance list goes here, I guess
        if self.data is None:
            self.preparePool(context, self.original)
        else:
            self.actionCollector = []
            self._header = ""
            self._footer = ""
            self.selectedColumn = ''
            self.fullHeaderWidth = 500
            ## do the slicing and batching here
            self.getPrefs(context)
            ipp = self.itemsPerPage
            renderers = [iquotient.ITabularRenderer(item) for item in islice(self.data, 0, ipp)]
            self._contents = [
                div(renderer=directive("anItem"), data=item) for item in renderers
            ]
            self.collectColumns(renderers)
            self.collectActions(renderers)
            self.calculateHeaders()
            self._header = self.headers

#            self.prepareList(context, self.data)

        actions = []
        for action in self.actionCollector:
            a = Action(action)
            setattr(self, 'bind_%s' % action, lambda context, theA = a: theA)
            def theAction(items, theAct = action):
                pool = self.original
                for iid in items:
                    itm = pool.store.getItemByID(int(iid))
                    getattr(iquotient.ITabularRenderer(itm), 'action_%s' % theAct)(pool)
            setattr(self, action, theAction)
            actions.append(a)

        footer = self._footer
        ## triple BARF
        for item in self._contents:
            if hasattr(getattr(item, 'data', None), 'render_bayes'):
                footer = (footer, invisible(data=self.original)[fileSpam])
                break

        self.Table = self._contents
        return formless.Property(
            'Table',  ## The property to get the list of stuff from
            formless.List(
                actions=actions,
                label="Table",
                header=self._header,
                footer=footer,
            )
        )

    def render_anItem(self, context, data):
        ## Put the tabular pool instance, which was passed to us in data, on the stack as a renderer
        ## so we can find render_ methods
        context.remember(data, IRendererFactory)

        selectedColumn = self.selectedColumn
        item = data.original
        if hasattr(data, 'getCellClass'):
            cellclass = data.getCellClass(context, item)
        else:
            cellclass=''

        if hasattr(item, 'storeID'):
            checkboxId = item.storeID
        else:
            checkboxId = 0
        return div(
            style="clear: both; height: 0.25em; width: %spx" % self.fullHeaderWidth)[' '], div(
                style="width: %spx" % self.fullHeaderWidth)[
            span(style=CHECKBOX)[
                input(
                    _class="freeform-list-item-checkbox",
                    type="checkbox",
                    name = "Table",
                    value = checkboxId,
                    onclick=freeform.LOCATE_AND_CHANGE_CURRENT_SELECTION_INPUT)
            ],
            [
                span(style=CRAP % width, _class=cellclass or 'cell-normal')[
                    invisible(renderer=directive(column), data=item)
                ]
                for position, column, width, displayName in self.columnCollector
            ],
        ], div(style="clear: both; height: 0.25em; border-bottom: 1px dashed #efefbf")[ ' ' ]

    def getPrefs(self, context):
        ava = context.locate(resource.IResource).avatar
        try:
            prefs = IPrefs(ava)
            self.itemsPerPage = itemsPerPage = prefs.getPref('itemsPerPage', 25)
            self.showInvisible = showInvisible = prefs.getPref('showInvisible')
        except components.CannotAdapt:
            self.itemsPerPage = itemsPerPage = 25
            self.showInvisible = showInvisible = True

    def locateIndexIterator(self, context, data):
        self.getPrefs(context)
        request = context.locate(IRequest)
        defaultIndex = data.suggestedIndex
        index = request.args.get('index', [defaultIndex])[0]
        self.index = index
        self.selectedColumn = index
        if index == defaultIndex:
            defaultQueryBack = data.suggestedBackwards
        else:
            defaultQueryBack = False

        queryBack = request.args.get('direction',
            [defaultQueryBack and 'backwards' or 'forewards'])[0] == 'backwards'

        if not data.indexes.has_key(index):
            index = 'uid'
        self.queryBack = queryBack

        ssk = request.args.get('start', [None])[0]
        startItemID = 0
        if ssk:
            startKey = data.indexes[index].sortableStringToKey(ssk)
            startItemID = int(request.args.get('startid',[0])[0])
        else:
            startKey = None
        request.getSession().setComponent(ILastPool, (data, index, queryBack))
        indexIterator = data.queryIndex(index, startKey=startKey, backwards=queryBack, startItemID=startItemID)

        self.indx = indx = []
        for item in islice(indexIterator, 0, self.itemsPerPage):
            renderer = iquotient.ITabularRenderer(item, None)
            if renderer is None:
                renderer = TabularNothing(item)
            indx.append(renderer)

        for nextMessage in indexIterator:
            startstring = data.indexes[index].keyToSortableString(nextMessage.getIndexKey(data, index))
            startidstring = str(nextMessage.storeID)
            next = href(data.storeID, start=startstring, index=index, startid=startidstring)['Next']
            break
        else:
            next = span(_class='disabled')['Next']
        if startKey is None:
            bList = []
        else:
            backIter = data.queryIndex(index, result=STORE_ID, startKey=startKey, startItemID=startItemID, backwards=not queryBack, limit=self.itemsPerPage+1)
            bList = list(backIter)
        # make sure we're not showing the first page
        if bList and ((not self.indx) or (bList[-1] != self.indx[0].original.storeID)):
            sid = bList[-1]
            obj = self.original.store.getItemByID(sid)
            endstring = data.indexes[index].keyToSortableString(obj.getIndexKey(data, index))
            prev = href(data.storeID, start=endstring, startid=sid, index=index)['Previous']
        else:
            prev = span(_class='disabled')["Previous"]

        nextPrev = lambda headerWidth: div(_class="batching-line", style="width: %spx" % headerWidth)[ span(_class="batching-prev") [ prev ], span(_class="batching-next") [ next ], xml('&nbsp;'), taglib.br ]

        return indexIterator, nextPrev

    def collectColumns(self, indx):
        self.columnCollector = columnCollector = []
        seenCols = {}
        self.fullHeaderWidth = 25
        for item in indx:
            accumulator = {}
            reflect.accumulateMethods(item, accumulator, 'render_')
            for name, method in accumulator.items():
                position = getattr(method, 'position', False)
                if position is False:
                    continue
                if getattr(method, 'default', False):
                    if not seenCols.has_key(name):
                        seenCols[name] = True
                        columnCollector.append((position, name, method.width, method.colname))
                        self.fullHeaderWidth += method.width
        columnCollector.sort()

    def collectActions(self, indx):
        self.actionCollector = actionCollector = []
        seenActions = {}
        for item in indx:
            accumulator = {}
            reflect.accumulateMethods(item, accumulator, 'action_')
            for name, method in accumulator.items():
                if not seenActions.has_key(name):
                    seenActions[name] = True
                    actionCollector.append(name)
        actionCollector.sort()

    def calculateHeaders(self, index='', queryBack=False, storeID=None, indexes={}):
        self.headers = headers = [span(style=CHECKBOX, _class='pool-header')[ ' ' ]]
        for position, column, width, displayName in self.columnCollector:
            if column == index:
                klas = 'pool-header-highlight'
                direction = queryBack and 'forewards' or 'backwards'
                extra = img(src="/images/icons/%s.png" % (queryBack and 'backwards' or 'forwards'))
            else:
                klas = 'pool-header'
                direction = queryBack and 'backwards' or 'forewards'
                extra = ''
            if storeID is not None and indexes.has_key(column):
                content = href(
                    storeID, index=column, direction=direction, start=''
                )[displayName, extra]
            else:
                content = displayName
            headers.append(
                span(style=CRAP % width, _class=klas)[content]
            )

    def preparePool(self, context, data):
        indexIterator, nextPrev = self.locateIndexIterator(context, data)
        index = self.index
        indx = self.indx
        itemsPerPage = self.itemsPerPage
        showInvisible = self.showInvisible
        queryBack = self.queryBack


        self.collectColumns(indx)
        columnCollector = self.columnCollector

        self.collectActions(indx)
        actionCollector = self.actionCollector

        self.calculateHeaders(index, queryBack, data.storeID, data.indexes)
        headers = self.headers

        if showInvisible:
            contentsList = indx
        else:
            contentsList = [item for item in indx if not item.invisible]
        if contentsList:
            contentsList = [
                div(renderer=directive("anItem"), data=item) for item in contentsList
            ]
        else:
            contentsList = [ "No items." ]
            self.fullHeaderWidth = 500
            headers = ''

        nextPrev = nextPrev(self.fullHeaderWidth)

        ## set ivars based on what we just figured out from looking at the pool
        self._header = nextPrev, div(style="width: %spx" % self.fullHeaderWidth)[ headers ]
        self._contents = contentsList
            ##div(renderer=directive("anItem"), data=item) for item in contentsList
#        ]
        self._footer = nextPrev


## test use only


class ItemBrowser(root.Base):
    document = html[
    head[title["Item: ", name]],
    body[
        h3[name],
        pre[anIntrospector]
    ]
]
