# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.


import itertools
import os
import random
from time import time as now
import datetime
import sre

from atop import tpython

from twisted.web import server, resource, static
from twisted.python import util
from twisted.python import reflect
from twisted.python import components
from twisted.cred.checkers import ICredentialsChecker

from quotient import web
from quotient.web import webinterfaces
from quotient import taglib
from quotient.iquotient import ITabularRenderer
from quotient import timezones

import nevow
from nevow import formless
from nevow import freeform
from nevow import iwoven
from nevow import appserver

from nevow import iwoven
from nevow import renderer
from nevow import stan
from nevow.tags import *
from twisted.web.woven import guard


templatePath = util.sibpath(web.__file__, 'templates')

class RootCrudMixin:
    """I require ChildPrefixMixin. ONLY MIX ME IN IF YOU ARE REALLY THE ROOT, I
    WILL GO AWAY LATER.
    """

    def child_images(self, request):
        return static.File(os.path.join(templatePath, 'images'))

    def child_css(self, request):
        return static.File(os.path.join(templatePath, 'newskin.css'))

    def child_freeform_css(self, request):
        return static.File(util.sibpath(nevow.__file__, 'freeform-default.css'))

    def child_users(self, request):
        return UsersPage(self.original)

    def child_reset(self, request):
        from atop import credup
        from quotient import reset
        a = credup.preauth(self.original.store, "janitor")
        return resource.IResource(reset.IPasswordResetter(a))

class Base(renderer.Renderer):
    adaptsAvatar = False
    """If a base subclass adapts the avatar to IResource, adaptsAvatar should
    be true and the .avatar attribute will be set to the Avatar. If a Base subclass
    adapts an object which is not the avatar (such as a Pool, or a Powerup or an
    Item), it should set adaptsAvatar to False, and .avatar will be set to
    original.store.
    """
    def __init__(self, original):
        self.original = original
        if self.adaptsAvatar:
            self.avatar = original
        else:
            self.avatar = original.store
        renderer.Renderer.__init__(self)

    def render(self, request):
        request.setHeader("Content-type", 'text/html; charset=UTF-8')
        # make this self.avatar.transact
        return self.original.store.transact(renderer.Renderer.render, self, request)

def domainFromHeader(store, request):
    """Retrieve host information and identify
    """
    domain = request.getHeader("host")
    if ':' in domain:
        domain = domain.split(':')[0]
    return domain

class UsersPage(Base):
    document = taglib.page("Users Directory",anonymous=True)[
        "Nothing to see here, sorry!"
        ]

    def getChild(self, path, request):
        if not path:
            return self
        from atop.faraway import IFarIndex
        localpart = path
        domain = domainFromHeader(self.original.store, request)
        return UserPage(localpart, domain)


class UserPage(Base):
    isLeaf = True
    def render(self, request):
        def _wf(data):
            request.write(data)
            request.finish()
        from atop.faraway import IFarIndex
        IFarIndex(self.original).farByName(localpart, domain).far(
            IPublicResource, 'renderPublic', request).addCallback(_wf)
        return server.NOT_DONE_YET


class QuotientFormPoster(appserver.FormPostGenerator):
    def postForm(self, request, res, configurable, ctx, bindingName, args):
        try:
            return res.avatar.transact(configurable.postForm, ctx, bindingName, args)
        except freeform.ImproperInputError:
            ## Hack: try calling ICommander.postForm, then call ICommander.command
            from quotient import commander
            if not args.has_key('input'):
                raise
            inp = args['input'][0]
            cmd = commander.ICommand(res.avatar)
            result = res.avatar.transact(cmd.commandRaw, request, inp)
            request.setComponent(iwoven.IHand, result)
            return res


pat = sre.compile(' \w+::')


class CommandLineGenerator(components.Adapter):
    """command format
    
    /methodName arg kw::hello kw2::world
    """
    def getResourceForRequest(self, request):
        if request.postpath[-1] == 'quotient_command_line':
            userinput = request.args['input'][0]
            if userinput.startswith('/'):
                #del request.args['input']
                userinput[1:].split()[0]
                methodName = userinput[1:].split()[0]
                request.postpath[-1] = 'freeform_post:%s' % methodName

                rest = userinput[len(methodName) + 1:]
                if not rest:
                    return
                ## rest is a set of keyword arguments deliniated by ::
                ## eg key::value can have spaces anotherKey:: anotherValue (note space between colons and value)
                ## -> {'key': ['value can have spaces'], 'anotherKey': [' anotherValue (not space between colons and value)']}
                ## question: should we lstrip the value to get rid of the space? or strip the value so that spaces don't matter?
                
                ## '----' represents the direct object, the one argument which can be to the right
                ## of the method name and doesn't have to be a keyword argument (although it can be)
                end = None
                for match in pat.finditer(rest):
                    prevend = end
                    start, end = match.start(), match.end()
                    if prevend is None:
                        do = rest[:start]
                        if do:
                            request.args['----'] = [do]
                    else:
                        request.args.setdefault(arg, []).append(rest[prevend:start])
                    arg = rest[start+1:end-2]
                if end is None:
                    request.args['----'] = [rest]
                else:
                    request.args.setdefault(arg, []).append(rest[end:])
                ## Now we have mutated the request so that the freeform form post parser will handle it!
            else:
                request.postpath[-1] = 'freeform_post:*' # it is impossible to have a nice day


class QuotientSite(appserver.NevowSite):
    generatorFactories = [
        CommandLineGenerator,
        QuotientFormPoster,
        appserver.PrettyPrinter,
    ]


class NoneContainer(components.Adapter):
    """For debugging purposes, let's implement IContainer for None;
    calls to child(context, name) will return an error string
    """
    __implements__ = iwoven.IContainer,

    def child(self, context, name):
        return "Child '%s' was requested, but the data on the stack was None." % name
components.registerAdapter(NoneContainer, type(None), iwoven.IContainer)


class IUserSignup(formless.TypedInterface):
    def signup(self,
        request=formless.Request(),
        username=formless.String(),
        password=formless.Password(),
        realname=formless.String()):
        """Create account

        Create a new account on this server.
        """
        pass
    signup = formless.autocallable(signup)


class AnonymousRoot(Base, RootCrudMixin, freeform.Configurable):
    document = [
    xml('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">\n'),
    html(xmlns="http://www.w3.org/1999/xhtml", lang="en", **{'xml:lang': "en"})[
        head[
            title["Divmod Quotient"],
            link(rel="stylesheet", href="/css"),
        ],
        body[
            div(style="border: 1px dotted black; margin: 1em auto; height: 480px; width: 640px;")[
            #div(style="border: 1px dotted black; margin: 1em auto; height: 1024px; width: 1280px; background-image: url(/images/logos/BgBg.png)")[
                div(style="margin: 2em")[
                    h1["Divmod Quotient"],
                    div(renderer=directive('welcome')),
                    form(action=guard.INIT_PERSPECTIVE)[
                        input(type='text',name='username'),
                        input(type='password',name='password'),
                        input(type='submit',name='hello'),
                    ],
                    div(renderer=directive('reset')),
                    p[a(href='signup')['Sign Up a New User']],
                    p[a(href='claim')['Claim a Ticket']],
                    freeform.configureIfPossible,
                ]
            ]
        ]
    ]
]

    def render_welcome(self, context, data):
        return 'Welcome! Please log in.'

    def render_reset(self, context, data):
        return a(href="/reset/")[ "Forgot your password?" ]

    def child_signup(self, request):
        return SignupPage(self.original)

    def child_claim(self, request):
        from quotient.web.powerup import DefaultPowerupDisplay
        from quotient.signup import IBoxOffice
        return ClaimPage(IBoxOffice(self.original.store))

    postLocation = ''

    storeID = ''

def afterPostView(request, item):
    """For use in form-post methods.  After a post, view the given item.

    Useful for redirecting after a user logs in.
    """
    url = '/'+('/'.join(request.sitepath + ['me', str(item.storeID)]))
    request.setComponent(freeform.IRedirectAfterPost, url)

class SignupPage(Base):
    __implements__ = IUserSignup
    document = taglib.page("Divmod Quotient", anonymous=True)[
        freeform.configure
        ]

    def signup(self, request, username, password, realname):
        from quotient import tap2
        from quotient.prefup import IPrefs
        from quotient import sysup
        try:
            ava = sysup.makeUser(self.original.store, username, password, tap2.POWERUPS, True)
        except sysup.SignupDisallowed, e:
            request.write("Signup disallowed! " + str(e))
        else:
            cc = ICredentialsChecker(self.original.store)
            topDomain = cc.getDomain().name
            p = IPrefs(ava)
            p.setPref('realname', realname)
            p.setPref('replyto', '%s@%s' % (username, topDomain))
            s = request.getSession()
            res = resource.IResource(ava)
            s.setResourceForPortal(res, s.guard.resource.portal, res.logout)
            afterPostView(request, p)
            request.setComponent(freeform.IRedirectAfterPost, "/me/%s" % p.storeID)

    postLocation = 'signup'

    storeID = ''

class ClaimPage(Base):
    document = taglib.page(title="Claim a Ticket", anonymous=True)[
        freeform.configure
        ]

class UserFrontPage(AnonymousRoot):
    def render_welcome(self, context, data):
        return p[
            'Welcome, ', self.original.name, '! You may visit your home page ', 
            a(href="me/")["here."], 
            ' If you are not ', self.original.name, ', please log in below.'
        ]


class NotRenderable(Base):
    document = html[head[title["Too Bad"]], body["I don't know how to render ", str, "."]]


class Me(Base):
    adaptsAvatar = True

    def render_visible(self, context, data):
        from quotient import prefup
        from quotient.web import pools
        pl = prefup.IPrefs(self.original).getPrimaryNavPool()
        return invisible(data=pl)[pools.PoolBrowser(pl, suggestedView="Gallery")]

    document = taglib.page(title="Quotient")[
        render_visible,
    ]

    def logout(self):
        print "LOGGED OUT"

    def getChild(self, name, request):
        if name == '':
            return self
        try:
            i = int(name)
            # get something from the db
            item = self.original.getItemByID(i)
            res = resource.IResource(item,default=None)
            if res is None:
                return NotRenderable(item)
            else:
                return res
        
        except ValueError:
            pass
        return Base.getChild(self, name, request)

    def child_compose(self, request):
        from quotient.web import compose
        return compose.Compose(self.original)


counter = itertools.count().next


def column(method, default=False, width=200, name=None):
    method.position = counter()
    method.default = default
    method.width = width
    indname = ' '.join(map(str.capitalize, method.__name__.split('_')[1:]))
    if name is None:
        name = indname
    method.indexname = indname
    method.colname = name


class TabularBase(components.Adapter, renderer.RenderFactory):
    __implements__ = ITabularRenderer,

    invisible = property(lambda self: getattr(self.original, 'invisible', False))

    def render_received(self, context, data):
        from quotient import prefup
        dr = data.dateCreated
        # convert to datetime object
        local = tpython.mkgmtime(dr)
        dtobj = datetime.datetime.fromtimestamp(local, timezones.Local)
        # and here's where we have to respect the user's account setting, hoorj
        try:
            res = context.locate(resource.IResource)
            astz = dtobj.astimezone(prefup.IPrefs(res.avatar).timezone)
        except components.CannotAdapt:
            astz = dtobj
        return astz.strftime('%Y/%m/%d %H:%M:%S')
    column(render_received, True, 100, 'Date')

    def render_name(self, context, data):
        if data.name is None:
            name = "%s (Unnamed)" % data.__class__.__name__
        else:
            name = taglib.headerToUTF8(data.name)
        if not hasattr(data, 'storeID'):
            return name
        return a(href=data.storeID)[name]
    column(render_name, True, 300)

noThing = TabularBase(None)


FourOhFour = renderer.FourOhFour

