# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.


from twisted.web.microdom import lmx
from twisted.web.microdom import Element, Comment, Text
from twisted.web import domhelpers


"""Code which can take an incoming DOM tree and "scrub" it clean, removing
all tag attributes which could potentially be harmful, and turning potentially
harmful tags such as script and style tags into div tags.
"""


def iternode(n):
    """Iterate a node using a pre-order traversal, 
    yielding every Element instance.
    """
    if isinstance(n, Element):
        yield n
    newChildNodes = None
    for c in n.childNodes:
        if isinstance(c, Comment):
            if not newChildNodes:
                newChildNodes = n.childNodes[:]
        else:
            for x in iternode(c):
                yield x
    if newChildNodes:
        n.childNodes = newChildNodes

# GAWD this is cheap
# remind me to put it in a new release of Twisted at some point

def setTagName(self, tagName):
    self.endTagName = self.nodeName = self.tagName = tagName

Element.setTagName = setTagName

_goodHtml = {
    'a': ['href'],
    'b': [],
    'div': [],
    'i': [],
    'strong': [],
    'em': [],
    'hr': [],
    'font': ['size'],
    'br': [],
    'ul': [],
    'li': [],
    'p': [],
    'table': ['width', 'height'],
    'tr': [],
    'td': ['width', 'height'],
    'img': ['width', 'height', 'src'],
    'form': ['action', 'method'],
    'input': ['type', 'name', 'value']
    }

_mutateHTML = {
    'html': ('div', []),
    'head': ('div', []),
    'body': ('div', []),
    'meta': ('span', []),
    'title': ('div', []),
}

def spanify(node):
    node.attributes = {}
    node.childNodes = []
    node.endTagName = node.nodeName = node.tagName = 'span'
    lnew = lmx(node).span()
    lnew.node.clean = True
    return lnew

def _handleIMG(node):
    ## TODO: Pass some sort of context object so we can know whether the user
    ## wants to display images for this message or not
    # del node.attributes['src'] #  = '/images/missing.png'
    l = spanify(node)
    l['style']='background-color: red; color: white; font-weight: bold'
    l.text('[ I M A G E ]')
    return node

_handleHTML = {
    'img': _handleIMG
}


def scrub(node):
    """Remove all potentially harmful elements from the node and return a wrapper
    node.  (WARNING: THIS MUTATES THE INPUT FOR SPEED.)
    """
    if node.nodeName == 'html':
        filler = body = lmx().div(_class="message-html")
        for c in node.childNodes:
            if c.nodeName == 'head':
                for hc in c.childNodes:
                    if hc.nodeName == 'title':
                        body.div(_class="message-title").text(domhelpers.gatherTextNodes(hc))
                        break
            elif c.nodeName == 'body':
                filler = body.div(_class='message-body')
                break
    else:
        filler = body = lmx().div(_class="message-nohtml")
    for e in iternode(node):
        if getattr(e, 'clean', False):
            # If I have manually exploded this node, just forget about it.
            continue
        ennl = e.nodeName.lower()
        if _goodHtml.has_key(ennl):
            if _handleHTML.has_key(ennl):
                e = _handleHTML[ennl](e)
            newAttributes = {}
            oldAttributes = e.attributes
            e.attributes = newAttributes
            for attr in _goodHtml[ennl]:
                if oldAttributes.has_key(attr):
                    newAttributes[attr] = oldAttributes[attr]
        elif _mutateHTML.has_key(ennl):
            newTagName, attrs = _mutateHTML[ennl]
            newAttributes = {}
            oldAttributes = {}
            e.attributes = newAttributes
            e.setTagName(newTagName)
            for attr in attrs:
                if oldAttributes.has_key(attr):
                    newAttributes[attr] = oldAttributes[attr]
        else:
            e.attributes.clear()
            e.setTagName("div")
            e.setAttribute("class", "message-html-unknown")
            div = Element('div')
            div.setAttribute('class', 'message-html-unknown-tag')
            div.appendChild(Text(ennl))
            e.childNodes.insert(0, div)
    filler.node.appendChild(node)
    return body.node

