# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

import sre
import sys

from twisted.web import resource
from twisted.python import urlpath

from quotient.web.webinterfaces import IRedirectAfterLogin


class RedirectAfterLogin(resource.Resource):
    def render(self, request):
        existing = request.getSession().getComponent(IRedirectAfterLogin, None)
        if existing is not None:
            url = str(existing)
            request.redirect(url)
            request.getSession().setComponent(IRedirectAfterLogin, None)
        else:
            url = str(
                urlpath.URLPath.fromRequest(request).sibling(
                    'me'
                ).child(
                    ''
                )
            )

            request.redirect(url)
        return '<html><body><a href="%s">Click here</a></body></html>' % url


def loginCallback(model):
    return RedirectAfterLogin()



class BadLogin(resource.Resource):
    def render(self, request):
        return """This is a placeholder page which tells you you logged in incorrectly"""


def badLoginCallback(model):
    return BadLogin()


def calculateSubmodel(cur):
    sml = []
    while cur.parent is not None:
        sml.append(cur.name)
        cur = cur.parent
    sml.reverse()
    return "/".join(sml)


def deuhcode(s):
    """Convert from a string possibly encoded in quoted-printable
    to a (???) string. (It's not unicode yet, is it? so what is it?)
    """
    # Man what the heck
    found = sre.findall(r'=\?[^?]+\?[QqBb]\?[^?]+\?=', s)
    if not found:
        return unicode(s, sys.getdefaultencoding(), "replace")
    for item in found:
        _, charset, encoding, text, _ = item.split('?')
        if encoding in 'Qq':
            text = text.decode('quopri')
        else:
            text = text.decode('base64')
        s = s.replace(item, text.decode(charset))
    return s


def formatizeHeader(string):
    """Convert from a string to a utf-8 encoded unicode string
    
    (The original design of this function called for it to truncate long
    strings and ellipsize them, but it currently does not do this)
    """
    try:
        string = deuhcode(string).encode('utf8')
    except LookupError:
        string = "(Unknown encoding) " + string
    return string


