# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

"""Interface markers used by the Divmod web UI code.
"""


from twisted.python import components


class IDivmodSession(components.Interface):
    def get(key, default=None):
        """Get the given named preference from this session.
        """

    def getLocal(request, key, default=None):
        """Get a preference that was set with setLocal
        when the given request object had the same
        request.prepath as when setLocal was called.
        """

    def set(key, value):
        """Set the given named preference to the given value.
        """

    def setLocal(request, key, value):
        """Set a preference which is only visible when getLocal
        is called with a request object that has the same
        request.prepath as the request did when setLocal was
        called.
        """

    def handleRequest(request):
        """Update the user's preferences with any preference
        submissions from this request.
        """

    def locatePowerup(request, powerupClass):
        """Attempt to find a powerup of the given type. This will
        first look in the request to find any powerups that have
        already been looked up this request, then look through
        the current pool's powerups for a powerup of the given
        type.
        """


class IObjectCrumbs(components.Interface):
    """An interface on the request which keeps track of the list
    objects that were involved in the url traversal process
    for this request.
    """


class IPowerpack(components.Interface):
    """An application
    this interface will totally change soon
    """
    def pageClass(self):
        """Return the subclass of Page to use to render this application
        """

    def graphic(self):
        """Return the graphic to draw next to links to this application
        """

    def commandLine(self, line):
        """Execute a command and return a result if successful, None
        if nothing happened
        """

    root = property(doc="The object to use as the root model")


class ICommandLineResult(components.Interface):
    """The result of entering a command on the command line.
    If a string, will be shown to the user.
    
    If a Resource, will be rendered. 
    (dp Not implemented, maybe we don't need the Resource thing)
    """


class ICurrentHome(components.Interface):
    """The Home instance associated with the current request.
    """


class ICurrentPowerup(components.Interface):
    """The Powerup which is responsible for the current web page.
    """


class ICurrentAddable(components.Interface):
    """The addable class whose form we are currently rendering.
    """


class ICurrentPool(components.Interface):
    """The pool instance associated with the current request.
    """


class ISummary(components.Interface):
    """The Widget which knows how to render a summary of the
    data class it was registered for.
    """
    def generate(self, request, node):
        """Return a new node populated with a representation of
        the data class. If this is not available, __str__ will be used.
        """

    def __str__(self):
        """Return a short string summary for use identifying the
        object.
        """


class ICurrentSlice(components.Interface):
    """The current slice of objects being displayed.
    """


class IMostRecentlyViewedMessage(components.Interface):
    """A tuple which indicates the message a user most recently acted
    upon. Tuple of (pool, iteratorParkedAtMessage)
    """


class IURLMaster(components.Interface):
    def url(self, obj):
        """Create and return a URL for the given object
        """


class IURL(components.Interface):
    def __str__(self):
        """Create and return a URL for the adapted object
        """


class IRedirectAfterLogin(components.Interface):
    """The URLPath to which we will redirect after successful login.
    """

