# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

import os
import random
import datetime
from StringIO import StringIO

from quotient.taglib import *

from twisted.python import components
from twisted.internet import threads
from twisted.web.microdom import parseString
from twisted.web import static
from twisted.web import resource

from atop import sched
from atop import tpython

from quotient.web.scrubber import scrub
from quotient.web import root
from quotient.web.root import column
from quotient import timezones
from quotient import prefup
from quotient import mimemessage

from nevow.renderer import RenderFactory
from nevow.tags import *
from nevow.iwoven import IRequest
from quotient.iquotient import ITabularRenderer

class ISpamGuesses(components.Interface):
    """A dict of lists of storeids in the current web page
    and what the guess was. Like

    {'spam': [1,2,3], 'clean': [4,5,6]}
    """


def render_received(self, context, data):
    dr = data.dateReceived
    # convert to datetime object
    local = tpython.mkgmtime(dr)
    dtobj = datetime.datetime.fromtimestamp(local, timezones.Local)
    # and here's where we have to respect the user's account setting, hoorj
    astz = dtobj.astimezone(prefup.IPrefs(data.store).timezone)
    return astz.strftime('%Y/%m/%d %H:%M:%S')


class TabularMessage(components.Adapter, RenderFactory):
    invisible = False
    def getCellClass(self, context, message):
        if not message.flags & mimemessage.MIMEMessage.SEEN:
            return "cell-new"
        return ''

    def render_flags(self, context, message):
        ava = context.locate(resource.IResource).avatar

        prefs = prefup.IPrefs(ava)

        accountColors = prefs.getPref('accountColors', {})

        rv = message.receivedVia
        thenode = context.tag

        if not message.flags & mimemessage.MIMEMessage.SEEN:
            if accountColors.has_key(rv):
                colorNum = accountColors[rv]
            else:
                colorNum = len(accountColors) + 1
                while colorNum >= 10:
                    colorNum %= 10
                accountColors[rv] = colorNum
            thenode[ img(src="/images/flags/new_%s.png" % colorNum, title=rv) ]
        for attach in message.getAttachments():
            break
        else:
            attach = None
        if attach is not None:
            thenode[ img(src="/images/flags/attachments.png") ]
        if message.flags & mimemessage.MIMEMessage.ANSWERED:
            thenode[ img(src="/images/flags/replied.png", title=rv) ]
        if message.flags & mimemessage.MIMEMessage.FORWARDED:
            thenode[ img(src="/images/flags/forwarded.png") ]
        if message.flags & mimemessage.MIMEMessage.REDIRECTED:
            ## todo create flags png for this
            thenode[ img(src="/images/toolbar/redirect.png") ]
        return thenode
    column(render_flags, True, 60)

    def render_sender(self, context, data):
        h = data.getIndexKey(None,'person')
        d = headerToUTF8(data.nameAndAddress()[0])
        if h:
            return a(href=str(h.storeID))[d]
        else:
            return d
    column(render_sender, True, 160)

    def render_bayes(self, context, data):
        from quotient.cleanup import ISpambayes
        try:
            bayes = context.locate(ISpambayes)
        except KeyError:
            top = context
            while top.parent is not None:
                top = top.parent
            ava = context.locate(resource.IResource).avatar
            bayes = ISpambayes(data.store)
            top.remember(bayes, ISpambayes)

        pool, prob = bayes.guessPool(data)
        if pool is None:
            # Convention is that if the pool is None
            # the message was routed by rule, the
            # name of the rule is in prob
            return [
                span(_class='fact') [ prob ],
                ]
        try:
            guesses = context.locate(ISpamGuesses)
        except KeyError:
            top = context
            while top.parent is not None:
                top = top.parent
            guesses = {}
            top.remember(guesses, ISpamGuesses)
        guesses.setdefault(pool, []).append(data.storeID)
        currentGuessStyle = {
            'Clean': 'file-clean',
            'Spam': 'file-spam',
            'Unknown': 'spam-unknown'
        }[pool]

        return [
            a(_class="train-as-spam", href="/me/%s?spam=%s" % (bayes.storeID, data.storeID)) [ "<" ],
            span(_class=currentGuessStyle) [ '%0.2f' % prob ],
            a(_class="train-as-clean", href="/me/%s?clean=%s" % (bayes.storeID, data.storeID)) [ ">" ]
        ]
    column(render_bayes, True, 80, "Spam?")

    def render_subject(self, context, data):
        subj = data['subject'] or ''
        if not subj.strip():
            subj = "<no subject>"
        return a(href=data.storeID)[headerToUTF8(subj)]
    column(render_subject, True, 300)

    render_received = render_received
    column(render_received, True, 170)

    def render_size(self, context, data):
        size = data.size
        if size < 1024:
            return "%dB" % (size / 1024)
            return str(size) + 'b'
        if size < 1024 * 1024:
            return "%.1fK" % (size / 1024.0)
        if size < 1024 * 1024 * 1024:
            return "%.2fM" % (size / (1024.0 * 1024))
    column(render_size, True, 60)

    def action_delete(self, pool):
        from quotient import gcup
        trash = gcup.ICollector(pool.store).trashRef.getItem()
        trash.addItem(self.original)
        pool.removeItem(self.original)

    def action_markUnread(self, pool):
        print "mark unread; ", self.original.flags
        self.original.changeFlag(self.original.SEEN, set=False)
        print "done", self.original.flags

    def action_markRead(self, pool):
        print "mark read; ", self.original.flags
        self.original.changeFlag(self.original.SEEN)
        print "done", self.original.flags


def viewedMessage(context, data):
    data.changeFlag(mimemessage.MIMEMessage.SEEN)
    return ""

def messageSource(context, data):
    label = 'View Message Source'
    return a(_class="message-view-style", href='%s?source=1' % data.storeID)[label]

def messageBody(context, data):
    req = context.locate(IRequest)
    if req.args.get('source'):
        return pre[data.openFile().read()]

    try:
        part = data.getDisplayPart()
    except ValueError:
        dom = div(_class="disabled")['No text in body.']
    else:
        partdata = part.get_payload(decode=True)
        ctypep = part.getHeaderParams('content-type')
        if ctypep.has_key('charset'):
            try:
                partdata = partdata.decode(ctypep['charset']).encode('utf8')
            except LookupError, le:
                partdata = '%s:\n%s' % (le, partdata)

        if 'html' in part.get_content_type().lower():
            try:
                dom = parseString(partdata, beExtremelyLenient=True)
            except:
                return 'Garbage HTML, sorry.'
            if req.args.get('trusted'):
                dom = xml(dom.toprettyxml())
            else:
                scrub(dom)
                dom = (a(_class="message-view-style", href='%s?trusted=1' % data.storeID)['Trust this HTML.'],
                       xml(dom.toprettyxml()))
        else:
            dom = pre[partdata]
    return dom

def messageExtracts(context, data):
    ava = context.locate(resource.IResource).avatar
    result = []
    if hasattr(data, 'extracts'):
        for extract in data.extracts:
            extract = extract.getItem()
            if extract.uninteresting:
                continue
            r = [extract.excerpt(15)[0]]
            result.append(p['Found:', r])
    return result

from quotient.web.pools import ILastPool
from atop.store import STORE_ID
import operator

def _link(text, previous):
    def _(context, data):
        req = context.locate(IRequest)
        # figure out which pool and index I'm iterating.
        p = req.getSession().getComponent(ILastPool, default=None)
        if p is None:
            pk = data.pools.keys()[0]
            pool = data.store.getItemByID(pk)
            indexName = pool.suggestedIndex
            lbackwards = pool.suggestedBackwards
        else:
            pool, indexName, lbackwards = p
        # TODO: distinction between index prefix and index name??
        skey = data.getIndexKey(pool, indexName)
        q = pool.queryIndex(indexName, startKey=skey, startItemID=data.storeID,
                            backwards=(previous ^ lbackwards), result=STORE_ID)
        for sid in q:
            break
        for sid in q:
            return a(href=str(sid))[text]
        else:
            return span(_class="disabled")[text]
    return _


def _action(text):
    def _(context, data):
        return a(href="/me/compose?%s=%s" % (text, data.storeID)) [ text ]
    return _


MODULE_NOT_PRESENT = "bad"


def attachments(context, msg):
    attList = list(msg.getAttachments())

    ## Look for images to make thumbnails out of.
    ## dp 11/5/03 move this somewhere else soon
    if attList:
        thumbs = ''
        pil = None
        for att in attList:
            inf = att.inferType()
            if inf.startswith('image'):
                if thumbs == '':
                    ## lazily create the thumbnail dom
                    thumbs = div(_class="attachmentButton")[
                        p["Thumbnails!"]
                    ]

                if pil is None:
                    ## lazily load pil
                    try:
                        from PIL import Image
                        pil = Image
                    except ImportError:
                        pil = MODULE_NOT_PRESENT

                if pil is MODULE_NOT_PRESENT:
                    ## No PIL available
                    thumbs[ img(src='%s/%s' % (msg.storeID, att.mimeID), style="height: 100px; width: 100px") ]
                else:
                    def _(attachment):
                        return img(src='%s/thumbsup:%s.png' % (msg.storeID, att.mimeID))
                    thumbs[ _(att) ]

        return div(_class="attachmentList")[
            "Attachments!", [
                div(_class="attachmentButton")[
                    a(href='%s/%s' % (msg.storeID, att.mimeID))[att.getAttachmentName()]
                ]
                for att in attList
             ],
             thumbs
        ]
    else:
        return ''

def From(context, data):
    pRef = data.getIndexKey(None, 'person')
    if pRef:
        return a(href=str(pRef.storeID))[messageHeader('From')]
    else:
        return messageHeader('From')

From.name = 'From'

class MessageRenderer(root.Base):
    document = page(title=(messageHeader('subject')))[
        div(_class="toolbar-line")[
            span(_class="toolbar-button")[ _action("reply") ],
            span(_class="toolbar-button")[ _action("forward") ],
            span(_class="toolbar-button")[ _action("redirect") ],
        ],
        div(_class="batching-line")[
            span(_class="batching-prev")[_link("Previous", True)],
            span(_class="batching-next")[ _link("Next", False) ]
        ],
        headers(From,'To','Cc','Date','Subject'),
        attachments,
        messageSource,
        messageBody,
        messageExtracts,
        viewedMessage,
]

    def getChild(self, path, request):
        if path.startswith('thumbsup:'):
            tmpdir = self.original.newCacheDir('thumbsup')
            partid = path.split('.')[0].split(':')[-1]
            p = self.original.getPartByID(int(partid))
            return static.File(os.path.join(tmpdir, '%s.png' % partid), p.inferType())
        i = int(path)
        p = self.original.getPartByID(i)
        return static.Data(p.get_payload(True),
                           p.get_type())


class ScheduledEventTabularRenderer(components.Adapter, RenderFactory):

    invisible = False

    __implements__ = ITabularRenderer, RenderFactory.__implements__

    def render_name(self, context, data):
        return a(href=data.storeID)[data.__class__.__name__]
    column(render_name, True, 200)

    def render_nextrun(self, context, data):
        now = sched.IScheduler(data.store).clock()
        then = data.getIndexKey(None, "scheduled")
        if then is None:
            return "???"
        return tpython.humanInterval(then - now)
    column(render_nextrun, True, 200)

    def render_status(self, context, data):
        if self.original.status:
            return img(
                src="/images/icons/generic.png", title=str(self.original.status)
            ), a(href="%s/freeform_post:check" % self.original.storeID)[ 'Check now' ]
        return ''
    column(render_status, True, 200)
