
# -*- test-case-name: quotient.test.test_contacts -*-
#
# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

from time import gmtime
from time import time as now

from __future__ import generators
import rfc822, time, os
from quotient import util, picklejar, itemstore, itempool, formless
from quotient.filepile import DefaultSorter,DecimalSorter, symlink, readlink
from quotient.itempool import Pool, dateKeyer

from quotient.web import fragments

from twisted.python.components import registerAdapter
from twisted.web.woven import interfaces as woveninterfaces


class AppendAKA:
    def __init__(self, message, person, name):
        self.message = message
        self.personRef = person.referTo()
        self.name = name
        self.question = 'Add name %s to person %s as an AKA?' % (name, person.name)

    def yes(self):
        c = self.personRef.getItem()
        c.addContactMethod('aka', self.name)
        self.message.actions.remove(self)

    def no(self):
        self.message.actions.remove(self)


contactMethods = formless.Choice(choices=['email', 'sip', 'phone', 'cell', 'fax', 'address', 'aka'])


class IContactMethod(formless.TypedInterface):
    method = contactMethods
    value = formless.String()


class IPerson(formless.TypedInterface):
    name = formless.String()

    def newContactMethod(self, method=contactMethods, value=formless.String()):
        """Add Contact Method
        
        Add information about a method of reaching this person."""
        return IContactMethod
    newContactMethod = formless.autocallable(newContactMethod)

    def attachMessage(self, message=formless.Object()):
        """Attach a message
        
        Attach a message to this person
        """
    # turn this on for next release
    #attachMessage = formless.autocallable(attachMessage)

    contactMethods = formless.List()
    messagePool = formless.Table()


## This is kind of weak but it should work until someone thinks of a better idea
## quotient.items.phones will put some keys in this dict
_discoverers = {
}


class ContactMethod(object):
    __implements__ = IContactMethod,
    def __init__(self, person, method, value):
        self.personRef = person.referTo()
        self._method = method
        self._value = value

    def __str__(self):
        return "%s: %s" % (self.method, self.value)

    def subsume(self, other):
        """Try to eat another ContactMethod
        """
        pass

    def addMatch(self, msgref, partnum, bytebegin, byteend):
        pass
#        self.matches.append(Match(msgref, partnum, bytebegin, byteend))

    def _getMethod(self):
        return self._method

    def _setMethod(self, new):
        self._method = new
        ContactMethod._setValue(self, self._value)

    method = property(_getMethod, _setMethod)

    def _getValue(self):
        return self._value

    def _setValue(self, new):
        discoverers = _discoverers.get(self._method, [])
        newinstance = None
        def yielder():
            yield new
        person = self.personRef.getItem()
        provisioner = person.store.avatar.provisioner
        for disc in discoverers:
            for contactmethod in disc.discover(yielder(), person, provisioner):
                ## if we discover anything, we're done
                newinstance = contactmethod
                break
            if newinstance is not None:
                break
        else:
            ## If we didn't break, we didn't discover anything smart
            newinstance = ContactMethod(person, self._method, new)

        person.replaceContactMethod(self, newinstance)

    value = property(_getValue, _setValue)


def _findPrimary(whatMethod):
    def locate(self):
        ## TODO If something has been set as the "primary" whatMethod, return it now
        for meth in self.contactMethods:
            if meth.method == whatMethod:
                return meth.value
        return ''
    return locate


class Person(itemstore.Item):
    __implements__ = IPerson,

    def __init__(self, store, name):
        itemstore.Item.__init__(self, store)
        self._name = name
        self.contactMethods = []
        mp = Pool(store)
        mp.addIndex('date', dateKeyer)
        mp.addIndex('name')
        mp.addIndex('subject')
        mp.save()
        self.messagePoolID = mp.storeID
        self.dateCreated = self.dateModified = gmtime(now())

    def save(self):
        date = gmtime(now())
        self.removeFromIndexes('dateModified', self.dateModified)
        self.dateModified = date
        self.addToIndexes('dateModified', date)
        itemstore.Item.save(self)

    def _findName(self):
        if self._name is not None and self._name.strip():
            return self._name
        return self.email

    name = property(_findName)

    email = property(_findPrimary('email'))

    def newContactMethod(self, method, value):
        cm = ContactMethod(self, method, value)
        self.contactMethods.append(cm)
        ## trigger the smartness code
        cm._setValue(value)

    def addContactMethod(self, cm):
        for existing in self.contactMethods:
            if existing.subsume(cm):
                return
        self.contactMethods.append(cm)
        self.save()

    def removeContactMethod(self, cm):
        self.contactMethods.remove(cm)
        self.save()

    def replaceContactMethod(self, old, new):
        index = self.contactMethods.index(old)
        self.contactMethods[index] = new
        self.save()

    def addMessage(self, message):
        self.messagePool.addItem(message)

    def _getMessagePool(self):
        return self.store.getItemByID(self.messagePoolID)
    messagePool = property(_getMessagePool)

    def changeName(self, name):
        self._name = name
        self.addToIndexes('name',name)
        self.save()

    def index_name(self):
        return self.name

    def index_email(self):
        return self.email

    def index_dateCreated(self):
        return self.dateCreated

    def index_dateModified(self):
        return self.dateModified


class ContactCollector(util.Forgetter):
    #Everyone loves Big Brother!
    persistenceForgets = ['store']
    contextRemembers = [('store', 'store')]

    __implements__ = itempool.IMembrane

    chatty = 0

    def __init__(self, store, poolID):
        self.store = store
        self.poolID = poolID

    def itemMovedIn(self, pool, message):
        contact = self.collectContact(message)
        if contact:
            message.contactRef = contact.referTo()
            contact.addMessage(message)
            contact.save()
        else:
            message.contactRef = None
            return 
        message.save()
        from quotient.items import phones

        def messageIter():
            for part in message.walk():
                if part.inferType() == 'text/plain':
                    for line in part.getBody().xreadlines():
                        yield line

        for i in phones.USPhoneNumber.discover(messageIter(), contact, self.store.avatar.provisioner):
            collector = phones.USPhoneNumber.collector(message, contact, i)
            message.addAction(collector)
    itemRoutedIn = itemMovedIn

    def collectContact(self, message, header='From'):
        rawFrom = message[header]
        if rawFrom is None:
            return None
        self.chat('processing ' + rawFrom)
        al = rfc822.AddressList(rawFrom).addresslist
        if al:
            name, email = al[0]
        else:
            return None
        pool = self.store.getItemByID(self.poolID)
        try:
            c = pool.indexes['email'].getItemsFromKey(email).next()
            if (c.email != email):
                raise StopIteration
            self.chat("\tWe've seen this address before.")
        except (StopIteration, KeyError):
            self.chat('\tThis is a new address! Creating a new Contact.')
            c = Person(self.store, name)
            c.addContactMethod(ContactMethod(c, 'email', email))
            c.creationDate = time.time()
            pool.addItem(c)
            c.save()
            return c

        if name:
            self.chat('\tThe name is not blank.')
            if c.name:
                if name != c.name and (name.lower() not in [ n.value.lower() for n in c.contactMethods if n.method == 'aka' ]):
                    self.chat("\tThis contact already has a name and it isn't the new name.")
                    message.addAction(AppendAKA(message,c,name))
            else:
                self.chat("\tThis contact had no previous name so we're adding the current one.")
                c.changeName(name)
        return c
    
    def chat(self, msg):
        if self.chatty:
            print msg

    def itemMovedOut(self, pool, item):
        if not item.contactRef:
            return
        contact = item.contactRef.getItem()
        for old in contact.messagePool:
            if old.storeID == item.storeID:
                contact.messagePool.removeItem(old)
                item.contactRef = None
                item.save()
                break
        contact.save()
        if not len(list(contact.messagePool)):
            ## Delete this contact
            contacts = self.store.getItemByID(self.poolID)
            for poolID, uid in contact.pools:
                if poolID == self.poolID:
                    break
            else:
                raise RuntimeError, "We tried to remove a contact, but it didn't seem to be in the contacts pool!"
            contacts.removeItem(contact)
            item.contactRef = None
            item.save()
    itemRoutedOut = itemMovedOut


class IAddPerson(formless.TypedInterface):
    def addPerson(self, name=formless.String()):
        """Add Person

        Add a person to your address book.
        """
        return formless.Object(interface=Person)
    addPerson = formless.autocallable(addPerson)


from quotient.powerup import Powerup


class AddPersonPowerup(Powerup):
    __implements__ = IAddPerson,
    def __init__(self, store, contactsPoolId):
        self.store = store
        self.contactsPoolId = contactsPoolId

    def addPerson(self, name):
        c = Person(self.store, name)
        contactsPool = self.store.getItemByID(self.contactsPoolId)
        contactsPool.addItem(c)


from twisted.web.woven import model


class AddPersonModel(model.Wrapper): pass
registerAdapter(AddPersonModel, AddPersonPowerup, woveninterfaces.IModel)
registerAdapter(fragments.AddContact, AddPersonModel, woveninterfaces.IView)


