# -*- test-case-name: quotient.test.test_contacts -*-
#
# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

from __future__ import generators

from twisted.python import components

from quotient.util import iterateInReactor
from quotient import formless

from quotient.items import contacts

from quotient.provisioner import ETank
import re


pnre = re.compile(r'((?:\D|^)?:(\d\d\d)[ -\.])?(\d\d\d)[ -\.](\d\d\d\d)(?:\D|$)')

LINE_COST = 0.001


class CollectPhoneNumber:
    def __init__(self, message, contact, pn):
        self.message = message
        self.contactRef = contact.referTo()
        self.pn = pn
        self.question = "Phone number '%s' detected. Is this %s's number?" % (pn, contact.name or contact.email)

    def yes(self):
        c = self.contactRef.getItem()
        c.addContactMethod(self.pn)
        c.save()
        self.message.actions.remove(self)

    def no(self):
        self.message.actions.remove(self)


class IUSPhoneNumber(formless.TypedInterface):
    method = contacts.contactMethods
    number = formless.String()

    def call(self):
        """Call
        
        Dial this number"""
    call = formless.autocallable(call, action="Call")


class USPhoneNumber(contacts.ContactMethod):
    # notice how we DON'T implement ContactMethod.__implements__
    # this is a temp hack
    __implements__ = IUSPhoneNumber,

    _method = 'phone'

    def __init__(self, person, value, area, exchg, exten):
        contacts.ContactMethod.__init__(self, person, self._method, value)
        self.areaCode = area
        self.exchange = exchg
        self.extension = exten
    
    def call(self):
        return "Won't it be cool when that works?"

    ###############
    ## Discovery and collection
    ###############

    def match(klass, someText):
        """Decide whether a string is a US Phone number.
        Return a match object if so.
        """
        return pnre.search(someText)
    match = classmethod(match)

    def discover(klass, message, person, provisioner):
        """look in some pre-parsed structure for matches
        
        hopefully replaces findPhones being a function
        """
        needSave = False
        ## for now assume message is a string generator
        for someText in message:
            ## I'm just ignoring recording matches right now until we have a good strategy
            if not provisioner.deplete(ETank.EXTRACTION, LINE_COST, True):
                break
            mat = klass.match(someText)
            if mat is not None:
                begin, end = mat.span()
                _,area, exchg, extn = mat.groups()
                instance = klass(person, someText[begin:end], area, exchg, extn)
                yield instance
                needSave = True
        if needSave:
            person.save()
    discover = classmethod(discover)

    collector = CollectPhoneNumber

    ###############
    ## end discovery and collection
    ###############

    def subsume(self, other):
        """Try to 'eat' an incoming phone number, if it's similar enough to this one.
        """
        if (self.exchange == ipn.exchange and self.extension == ipn.extension):
            if ipn.areaCode:
                if self.areaCode:
                    if self.areaCode == ipn.areaCode:
                        doIt = True
                    else:
                        doIt = False
                else:
                    self.areaCode = ipn.areaCode
                    doIt = True
            else:
                doIt = True
        else:
            doIt = False
        if doIt:
            self.matches.extend(ipn.matches)
        return doIt

    def _calculateValue(self):
        return "(%s) %s-%s" % (self.areaCode, self.exchange, self.extension)

    def _setValue(self, new):
        klass = type(self)
        person = self.personRef.getItem()
        if not klass.match(new):
            person.replaceContactMethod(self, contacts.ContactMethod(person, self.method, new))
        else:
            def yielder():
                yield new
            pn = klass.discover(yielder(), person, person.store.avatar.provisioner).next()
            self.areaCode = pn.areaCode
            self.exchange = pn.exchange
            self.extension = pn.extension
            person.save()

    number = property(_calculateValue, _setValue)

    def __str__(self):
        return "phone: %s" % self.number


contacts._discoverers['phone'] = [USPhoneNumber]
contacts._discoverers['cell'] = contacts._discoverers['phone']
contacts._discoverers['fax'] = contacts._discoverers['phone']


#####
## old
#####

def findPhones(m, c):
    needSave = False
    avatar = c.store.avatar
    prov = avatar.provisioner
    for part in m.walk():
        if part.inferType().startswith('text/'):
            byteOfft = 0
            for line in part.getBody().xreadlines():
                if not prov.deplete(ETank.EXTRACTION, LINE_COST, True):
                    break
                yield None
                mat = pnre.search(line)
                if mat is not None:
                    begin, end = mat.span()
                    theMatchText = line[begin:end]
                    begin += byteOfft
                    end += byteOfft
                    area, exchg, extn = mat.groups()
                    pn = contacts.ContactMethod(c, 'phone', theMatchText)
                    IUSPhoneNumber(pn).remember(area, exchg, extn)
                    pn.addMatch(m, part.partnum, begin, end)
                    m.addAction(CollectPhoneNumber(m, c, pn))
                    needSave = True
                byteOfft += len(line)
    if needSave:
        c.save()
    avatar.save()


class Match:
    def __init__(self, item, partnum, bytebegin, byteend):
        self.msgref = item.referTo()
        self.partnum = partnum
        self.bytebegin = bytebegin
        self.byteend = byteend


class IPhoneNumber(formless.TypedInterface):
    value = formless.String()


class UserEnteredPhoneNumberHelper:
    """This sucks and doesn't work anyway
    
    terminate with extreme prejudice
    """
    def __init__(self, theString):
        self.theString = theString
        self.found = None

    def getBody(self, *args, **kw):
        return self

    def xreadlines(self):
        yield self.theString

    def inferType(self):
        return 'text/plain'

    def walk(self):
        yield self

    def addAction(self, cpn):
        import pdb; pdb.set_trace()
        self.found = cpn.pn
        

class PhoneNumber(object):
    __implements__ = IPhoneNumber,

    method = 'phone'

    def __init__(self, contact, area, exchg, exten):
        self.contactRef = contact.referTo()
        self.areaCode = area
        self.exchange = exchg
        self.extension = exten
        self.userValue = None
        self.matches = []

    def _calculateValue(self):
        print "GET VALUE"
        if self.userValue is None:
            return "(%s) %s-%s" % (self.areaCode, self.exchange, self.extension)
        return self.userValue

    def _setValue(self, new):
        print "SET VALUE", new
        uepnh = UserEnteredPhoneNumberHelper(new)
        cr = self.contactRef.getItem()
        findPhones(uepnh, cr)
        pn = uepnh.found
        if pn is not None:
            self.areaCode = pn.areaCode
            self.exchange = pn.exchange
            self.extension = pn.extension
            self.matches = pn.matches
            self.userValue = None
        else:
            self.userValue = new

    value = property(_calculateValue, _setValue)

    def addMatch(self, msgref, partnum, bytebegin, byteend):
        self.matches.append(Match(msgref, partnum, bytebegin, byteend))

    def subsume(self, ipn):
        """Try to 'eat' an incoming phone number, if it's similar enough to this one.
        """
        if (self.exchange == ipn.exchange and self.extension == ipn.extension):
            if ipn.areaCode:
                if self.areaCode:
                    if self.areaCode == ipn.areaCode:
                        doIt = True
                    else:
                        doIt = False
                else:
                    self.areaCode = ipn.areaCode
                    doIt = True
            else:
                doIt = True
        else:
            doIt = False
        if doIt:
            self.matches.extend(ipn.matches)
        return doIt

    def __str__(self):
        return "phone: %s" % self.value
    
    def save(self):
        self.contactRef.getItem().save()
