# -*- test-case-name: quotient.test.test_blog -*-
import itertools

from twisted.python import components

from quotient.items.powerup import Powerup, IPowerStation
from quotient.util import ItemForwardingComponent
from quotient.storq import Pool, UIDIndex, IPoolIndex, Item, ITEM

class IBloggable(components.Interface):
    """
    @ivar messageParentRef: Reference to this message's parent item
    """

    def addPost(self, item, parent=None):
        """Add a top-level message to this blog.
        """

    def removePost(self, item, andChildren=True):
        """Remove a message from this blog.
        """

    def getChildren(self, parent=None, N=None):
        """Retrieve the most recent N messages.
        """

def _formatItemPath(path):
    return '.'.join(['%10d' % e for e in path])

def _superMessageParentPath(msg):
    r = []
    p = IBloggable(key).parentMessageRef
    while p is not None:
        r.append(_formatItemPath(p.getItemPath()))
        p = IBloggable(p.getItem()).messageParentRef
    r.reverse()
    return r

class ParentIndex(UIDIndex):
    __implements__ = (IPoolIndex,)
    name = "parent"

    def keyToSortableString(self, key):
        return '-'.join(_superMessageParentPath(key))

    def sortableStringToKey(self, sstr):
        return int(sstr.split('-')[-1])

class BlogPoolAdapter(components.Adapter):
    __implements__ = (IBloggable,)

    def __init__(self, original):
        components.Adapter.__init__(self, original)
        original.addIndex(ParentIndex())
        self.messageParentRef = original.referenceTo()

    def addPost(self, item):
        blog = self.blogRef.getItem()
        blog.addItem(item)

    def removePost(self, item):
        blog = self.messageParentRef.getItem()
        for c in self.getChildren(item):
            self.removePost(c)
        blog.removeItem(item)

    def getChildren(self, N=None):
        blog = self.messageParentRef.getItem()
        path = _superMessageParentPath(self)

        # Everything up to the most deep parent
        # plus everything up to and excluding the most deep part of that parent's path
        # plus the most deep part of the parent's path plus one
        endKey = path[:-1] + [path[-1][:-1] + [path[-1][-1] + 1]]
        return list(blog.queryIndex('parent', result=ITEM, startKey=path, endKey=endKey))

class MIMEMessageBloggableAdapter(components.Adapter):

    __implements__ = IBloggable

    def __init__(self, original):
        components.Adapter.__init__(self, original)
        self.author = original['from']
        self.subject = original['subject']
        self.body = original.getDisplayPart()
        self.messageParentRef = original.referenceTo()

    def index_parent(self, pool):
        return self.parentRef.getItem()
