
from os.path import join as opj

from twisted.trial import unittest

from quotient.items import blog, powerup
from quotient import storq, storcred

class BlogTest(unittest.TestCase):
    def setUp(self):
        d = self.mktemp()
        self.store = storq.Store(opj(d, 'db'), opj(d, 'files'))
        def _():
            self.avatar = storcred.Avatar(self.store)
            self.pool = storq.Pool(self.avatar, name="blog")
            self.pool.setComponent(blog.IBloggable, blog.BlogPoolAdapter(self.pool))
            self.blog = blog.BlogPoolAdapter(self.pool)
        self.store.transact(_)

    def tearDown(self):
        self.store.close()

    def testAddPost(self):
        def _():
            m = blog.Message(self.avatar, self.pool, "bob", "this is my blog and it is nifty", "yes")
            self.blog.addPost(m)
            self.assertEquals(self.blog.getTopLevel(), [m])
        self.store.transact(_)

    def testAddMorePosts(self):
        def _():
            m = blog.Message(self.avatar, self.pool, "bob", "this is my blog and it is nifty", "yes")
            self.blog.addPost(m)
            m2 = blog.Message(self.avatar, m, "fred", "hey you, your blog sucks", "bla")
            self.blog.addPost(m2)
            self.assertEquals(self.blog.getTopLevel(), [m])
            self.assertEquals(self.blog.getChildren(m), [m2])
            self.assertEquals(self.blog.getChildren(m2), [])
        self.store.transact(_)

    def testRemoval(self):
        def _():
            m = blog.Message(self.avatar, self.pool, "bob", "this is my blog and it is nifty", "yes")
            self.blog.addPost(m)
            m2 = blog.Message(self.avatar, m, "fred", "hey you, your blog sucks", "bla")
            self.blog.addPost(m2)

            self.blog.removePost(m)

            self.assertEquals(self.blog.getTopLevel(), [])
            self.assertEquals(self.blog.getChildren(m), [])
            self.assertEquals(self.blog.getChildren(m2), [])
        self.store.transact(_)

    def testComplexerConfiguration(self):
        def _():
            m = blog.Message(self.avatar, self.pool, "bob", "this is my blog and it is nifty", "yes")
            self.blog.addPost(m)
            m2 = blog.Message(self.avatar, m, "fred", "hey you, your blog sucks", "bla")
            self.blog.addPost(m2)
            m3 = blog.Message(self.avatar, m2, "fred", "hey you, your blog sucks", "bla")
            self.blog.addPost(m3)
            m4 = blog.Message(self.avatar, m3, "fred", "hey you, your blog sucks", "bla")
            self.blog.addPost(m4)
            m5 = blog.Message(self.avatar, m3, "fred", "hey you, your blog sucks", "bla")
            self.blog.addPost(m5)
            m6 = blog.Message(self.avatar, m3, "fred", "hey you, your blog sucks", "bla")
            self.blog.addPost(m6)
            m7 = blog.Message(self.avatar, m6, "fred", "hey you, your blog sucks", "bla")
            self.blog.addPost(m7)
            m8 = blog.Message(self.avatar, m7, "fred", "hey you, your blog sucks", "bla")
            self.blog.addPost(m8)
            m9 = blog.Message(self.avatar, m8, "fred", "hey you, your blog sucks", "bla")
            self.blog.addPost(m9)

            self.assertEquals(self.blog.getTopLevel(), [m])
            self.assertEquals(self.blog.getChildren(m), [m2])
            self.assertEquals(self.blog.getChildren(m2), [m3])
            self.assertEquals(self.blog.getChildren(m3), [m4, m5, m6])
            self.assertEquals(self.blog.getChildren(m4), [])
            self.assertEquals(self.blog.getChildren(m5), [])
            self.assertEquals(self.blog.getChildren(m6), [m7])
            self.assertEquals(self.blog.getChildren(m7), [m8])
            self.assertEquals(self.blog.getChildren(m8), [m9])

        self.store.transact(_)
