'''
farByName's existence means that there is a global hierarchy of names for the
domain, somewhere.  when we register a user, we need to create a proto-ID; we
need to say "When I go to look up this user, I need to be able to use this ID".

This is a fairly simple distributed transaction, but it is a transaction
nonetheless.

the conversation really wants to be:
host -> client
 create yourself an avatar, ID 157, name 'bob'
host <- client
 okay, created it
host
 remembered ID 157

but that's going to be pretty error prone, because we could lose either the
'create yourself an avatar' message (what then?  timeout?) or the 'okay,
created it' message.

All of these error conditions are transient.  The host is not allowed to stay
down for good, so at some point it will be resolved, even if it requires
operator intervention.  If we lose the message on the way out, we keep
retransmitting until we receive an ack.  If we lose the ack, we keep
retransmitting until we get an ack-ack.

the createRemote call has to happen on the leaf-authoritative server.  so for
example if we have an authoritative server for the whole domain which delegates
responsibility for users with names starting A-C, the A-C server gets a
delegated createRemote, with the rid having been created on the master-master
server.

the createRemote call _starts_ on the master-master server though.
'''

class FarIndex:
    def activate(self):
        """
        'What rough beast, its hour come at last, slouches towards Bethlehem to
        be born...'

        Here is where we will do all the stuff related to the reactor: most of
        the work being done here will be initiation of persistent connections,
        which will make sure that this object is strong-referenced by the
        reactor and will live indeterminately.

        Luckily since we are faking the remoting now, we don't need to deal
        with these issues, but we should be considering that.
        """

class NoSuchObject(Exception):
    """No such object exists.
    """

class NameConflict(Exception):
    """There already exists a user with a specified name when trying to create.
    """
