#!/usr/bin/env python2.3 
import urllib, random, time

from twisted.trial import unittest, util
from twisted.internet import reactor, defer
from twisted.python import log, components
from twisted.application import service
from twisted.protocols import policies

import quotient
from quotient.web import newclient, functional
from quotient.test import simpleSpider

from flooder import Flooder, Tee
from torture_userCreation import SignupTaskFactory, SignupJob, TweakableObserver, \
                                 RandomUser



def _eb(error):
    log.err(error)
    return None     # swallow any errors


class TortureSpider(simpleSpider.SimpleSpider, Flooder):
    def __init__(self):
        Flooder.__init__(self)

    def seedUrls(self):
        doneSeeding = defer.Deferred()

        def eb(error):
            log.err(error)
            
        def updateDict(result):
            self.updateUrlDict(result)
            doneSeeding.callback(None)
        
        def getHome(_):
            d = self.doGet('/me/0')
            d.addCallback(updateDict)

        def signUp(_):
            d = self.doGet("/signup/freeform_post:signup", self.user.dictItems)
            d.addCallback(getHome)

        def _(**kw):
            #print 'seedUrls'
            d = self.doGet('/')
            d.addCallback(signUp)

        reactor.callLater(0, _)
        return doneSeeding


    def harvestUrls(self):
        done = defer.Deferred()
       
        def doHarvest(_):
            for url in self.urldict.values():
                if url in simpleSpider.dontGoThere:
                    continue
                if '/me' not in url:
                    d = self.doGet('/me/%s' % url)
                else:
                    d = self.doGet(url)

                d.addCallback(self.updateUrlDict).addErrback(_eb)
            self.fixUrls()
            done.callback(None)

        def _():
            d = self.seedUrls()
            d.addCallbacks(doHarvest, _eb)

        reactor.callLater(0, _)
        return done
        

class SpiderJob(SignupJob):
    def __init__(self, **kw):
        SignupJob.__init__(self, **kw)

    def doWork(self):
        jobDone = defer.Deferred()

        def done(_):
            jobDone.callback(None)

        def _():
            self.master.clockIn(self._jid)
            log.msg('SignupFlood.signup')
            log.msg('user:', self.user)
            ts = TortureSpider()
            ts.user = self.user
            ts.harvestUrls(
                    ).addCallback(done
                    ).addErrback(_eb)
            
        reactor.callLater(0, _)
        jobDone.addCallback(lambda _: self.master.clockOut(self._jid))
        return jobDone


class SpiderTaskFactory(SignupTaskFactory):
    def __init__(self):
        SignupTaskFactory.__init__(self)

    def getToWork(self):
        self.jobsDone = SignupTaskFactory.getToWork(self)
        return self.jobsDone

    def getAJob(self):
        j = SpiderJob(user=RandomUser())
        j.master = self
        j.baseurl = self.baseurl
        return j

    def allJobsCompleted(self):
        log.msg('allJobsCompleted!')
        log.msg("completed %s jobs in %s sec" % (self._totalCompletedJobs, time.time() - self.startTime))
        d, self.floodComplete = self.floodComplete, None
        d.callback(None)
#        reactor.stop()


from twisted.python import components
from twisted.spread import flavors
import task
components.registerAdapter(
        task.PBTaskFactoryAdapter,
        SpiderTaskFactory,
        flavors.IPBRoot
)


if __name__ == "__main__":
    import os, sys
    import os.path as osp
    logname = 'torture.log'
    o = TweakableObserver(sys.stdout)
    log.addObserver(o._emit)
    log.startLogging(file(logname,'w'), False)
    
    f = SpiderTaskFactory()
    f.totalJobsLimit = 30
    f.activeJobsLimit = 5 
    f.jobsPerSecond = 0.5

    reactor.callWhenRunning(f.getToWork)
        
    #import pdb
    #pdb.run("reactor.run()")
    
    reactor.run()


