from nevow.tags import *
import schedobj
import string
import re

DEBUG = False

daysOfWeek = ["Monday","Tuesday","wednesday","Thursday","Friday","Saturday","Sunday"]

class FormattedLine(object):
    def __init__(self, dateOrTime=None, team=None, open=None, id=None):
        self.dateOrTime = dateOrTime
        self.team = team
        self.open = open
        self.id = id

class Game(object):
    dateOrTime = [] 
    team       = []
    open       = []
    id         = []
    lines      = None

    def __init__(self, lines):
        '''lines is a (team1, team2) tuple of lines'''
        self.lines = lines
        try:
            self.getDateOrTime()
            self.getOpen()
            self.getTeams()
            self.getIDs()
        except Exception, e:
            print e
            print self
            raise

    def __str__(self):
        return """
lines: %s
dateOrTime: %s
team: %s
open: %s
id: %s
""" % (self.lines, self.dateOrTime, self.team, self.open, self.id)


    def getDateOrTime(self):
        self.dateOrTime = [self.lines[n][0:6].strip() for n in xrange(len(self.lines))]

    def getOpen(self):
        self.open = [self.lines[n][7:13].strip() for n in xrange(len(self.lines))]

    def getTeams(self):
        self.team = [self.lines[n][14:31].strip() for n in xrange(len(self.lines))]

    def getIDs(self):
        self.id = [self.lines[n][31:34] for n in xrange(len(self.lines))]

    def getLine(self, num):
        atr = tr()
        atr[
            td[self.dateOrTime[num]],
            td[self.open[num]],
            td[self.team[num]],
            td[self.id[num]]
        ]
        return atr
    
class Schedule(object):
    seplines  = []
    teamlines = []
    headlines = []
    datelines = []
    games = []

    def __init__(self, fileobj=None):
        self.fileobj = fileobj
            
    def createSched(self):
        self.findLines()
        self.createGames()
        self._logme()

    def _logme(self):
        logfile = file('sched.log', 'w')
        logfile.write('seplines\n%s' % self.seplines)
        logfile.write('\nteamlines\n%s' % self.teamlines)
        logfile.write('\nheadlines\n%s' % self.headlines)
        logfile.write('\ndatelines\n%s' % self.datelines)

        for game in self.games:
            logfile.write(str(game))

        logfile.close()

    def findLines(self):
        lines = self.fileobj.readlines()
        self.fileobj.close()
        
        num = 0
        while num < len(lines):
            if lines[num][0] == '-':
                self.seplines.append(num)
                if DEBUG:
                    print "sepline %d: %s" % (num, lines[num])
            elif lines[num][0] == ' ':
                self.headlines.append((num, num+1))
                for day in daysOfWeek:
                    if lines[num+1].find(day) != -1:
                        self.datelines.append(num+1)
                if DEBUG:
                    print "headline1 %d: %s\nheadline2 %d: %s" % (num, lines[num], num+1, lines[num+1])
            elif lines[num][0].isdigit():
                self.teamlines.append((num, num+1))
                if DEBUG:
                    print """team1line %d: %s\nteam2line %d: %s""" % (num, lines[num], num+1, lines[num+1])
                num += 1
            else:
                if DEBUG:
                    print "ignoreline %d: %s" % (num, lines[num])
            num += 1
        self.lines = lines

    def createGames(self):
        for t in self.teamlines:
            self.games.append(Game((self.lines[t[0]], self.lines[t[1]])))

    def renderMe(self, context, data):
        dv = div()
        for d in data:
            dv[
                tr[d.getLine(0)],
                tr[d.getLine(1)]
            ]
        return dv

if __name__ == "__main__":
    fobj = file('schedule.txt', 'rb')
    s = Schedule(fobj)
    s.createSched()

